﻿using System.Collections.Generic;
using System.Linq;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models.Interfaces;
using ODMWebService.DL.Entities.Enums;

namespace ODM.Backend.Models.Models
{
	public class Competition : IDbItem, ICompetition
	{
		public int Id { get; set; }
		public bool IsRemoved { get; set; }
		public CompetitionTypeEnum CompetitionType { get; set; }
		public string Discipline { get; set; }
		public Gender Gender { get; set; }
		public string Race { get; set; }
		public int? SportId { get; set; }

		public List<string> Cities { get; set; }

		public override string ToString()
		{
			if (Race == "")
			{
				return Discipline;
			}

			return $"{Discipline}: {Race}";
		}
	}

	public sealed class CompetitionComparer : IEqualityComparer<Competition>
	{
		public bool Equals(Competition x, Competition y)
		{
			if (x == null)
				return y == null;
			else if (y == null)
				return false;
			else
				return x.Id.Equals(y.Id)
				       && x.IsRemoved.Equals(y.IsRemoved)
				       && x.Race.Equals(y.Race)
				       && x.Gender.Equals(y.Gender)
				       && x.CompetitionType.Equals(y.CompetitionType)
				       && x.Discipline.Equals(y.Discipline)
				       && x.SportId.Equals(y.SportId)
					   && x.Cities.SequenceEqual(y.Cities);
		}

		public int GetHashCode(Competition obj)
		{
			unchecked
			{
				int hash = 17;

				hash = hash * 23 + (obj.Id).GetHashCode();
				hash = hash * 23 + (obj.IsRemoved).GetHashCode();
				hash = hash * 23 + (obj.Race ?? "").GetHashCode();
				hash = hash * 23 + (obj.Gender).GetHashCode();
				hash = hash * 23 + (obj.CompetitionType).GetHashCode();
				hash = hash * 23 + (obj.SportId).GetHashCode();
				hash = hash * 23 + (obj.Discipline ?? "").GetHashCode();
				foreach (var objCity in obj.Cities)
				{
					hash = hash * 23 + (objCity).GetHashCode();
				}

				return hash;
			}
		}
	}
}