﻿using System;
using System.Collections.Generic;
using System.Linq;

// ReSharper disable CheckNamespace

namespace ODM.Backend.Models.Models
{
	/// <summary>
	/// StaticAppData je model, který zapouzdřuje všechna data, která jsou v průběhu ODM neměnná
	/// </summary>
	public class StaticAppData
	{
		public int Version { get; set; }

		public List<Venue> Venues { get; set; } = new List<Venue>();
		public List<Meal> Meals { get; set; } = new List<Meal>();
		public List<Menu> Menus { get; set; } = new List<Menu>();
		public List<Region> Regions { get; set; } = new List<Region>();
		public List<Sport> Sports { get; set; } = new List<Sport>();

		/// <summary>
		/// Zaplní se při serializaci dat na straně serveru, data jsou vytahaná z Regionů
		/// </summary>
		public List<string> Cities { get; set; } = new List<string>();

		public StaticAppData Merge(StaticAppData data)
		{
			this.Version = data.Version;

			if (!this.Cities.SequenceEqual(data.Cities))
			{
				this.Cities = data.Cities;
			}

			var venuesDiff = data.Venues.Except(this.Venues, new VenueComparer());

			foreach (var venue in venuesDiff)
			{
				var local = this.Venues.Find(m => m.Id == venue.Id);
				if (local != null)
				{
					var index = this.Venues.IndexOf(local);
					this.Venues[index] = venue;
				}
				else
				{
					Venues.Add(venue);
				}
			}

			var mealDiff = data.Meals.Except(this.Meals, new MealComparer());

			foreach (var meal in mealDiff)
			{
				var local = this.Meals.Find(m => m.Id == meal.Id);
				if (local != null)
				{
					var index = this.Meals.IndexOf(local);
					this.Meals[index] = meal;
				}
				else
				{
					Meals.Add(meal);
				}
			}

			var menuDiff = data.Menus.Except(this.Menus, new MenuComparer());

			foreach (var menu in menuDiff)
			{
				var local = this.Menus.Find(m => m.Id == menu.Id);
				if (local != null)
				{
					var index = this.Menus.IndexOf(local);
					this.Menus[index] = menu;
				}
				else
				{
					Menus.Add(menu);
				}
			}

			var regionDiff = data.Regions.Except(this.Regions, new RegionComparer());

			foreach (var region in regionDiff)
			{
				var local = this.Regions.Find(m => m.Id == region.Id);
				if (local != null)
				{
					var index = this.Regions.IndexOf(local);
					this.Regions[index] = region;
				}
				else
				{
					Regions.Add(region);
				}
			}

			var sportDiff = data.Sports.Except(this.Sports, new SportComparer());

			foreach (var sport in sportDiff)
			{
				var local = this.Sports.Find(m => m.Id == sport.Id);
				if (local != null)
				{
					var index = this.Sports.IndexOf(local);
					this.Sports[index] = sport;
				}
				else
				{
					Sports.Add(sport);
				}
			}

			return this;
		}
	}
}