﻿using System;
using System.Collections.Generic;
using ODM.Backend.Models.Models.Interfaces;

namespace ODM.Backend.Models.Models
{
	public class Event : IEvent, IDbItem
	{
		public DateTime? End { get; set; }
		public string Info { get; set; }
		public DateTime Start { get; set; }
		public int VenueId { get; set; }
		public int CompetitionId { get; set; }
		public string Title { get; set; }
		public int Id { get; set; }
		public bool IsRemoved { get; set; }
	}

	public sealed class EventComparer : IEqualityComparer<Event>
	{
		public bool Equals(Event x, Event y)
		{
			if (x == null)
				return y == null;
			else if (y == null)
				return false;
			else
				return x.Id.Equals(y.Id)
				       && x.IsRemoved.Equals(y.IsRemoved)
				       && x.Info.Equals(y.Info)
				       && x.CompetitionId.Equals(y.CompetitionId)
				       && x.End.Equals(y.End)
				       && x.Start.Equals(y.Start)
				       && x.VenueId.Equals(y.VenueId)
					   && x.Title.Equals(y.Title);
		}

		public int GetHashCode(Event obj)
		{
			unchecked
			{
				int hash = 17;

				hash = hash * 23 + (obj.Id).GetHashCode();
				hash = hash * 23 + (obj.IsRemoved).GetHashCode();
				hash = hash * 23 + (obj.Info ?? "").GetHashCode();
				hash = hash * 23 + (obj.CompetitionId).GetHashCode();
				hash = hash * 23 + (obj.End).GetHashCode();
				hash = hash * 23 + (obj.Start).GetHashCode();
				hash = hash * 23 + (obj.VenueId).GetHashCode();
				hash = hash * 23 + (obj.Title).GetHashCode();

				return hash;
			}
		}
	}
}