﻿using System;
using System.Collections.Generic;
using System.Net.Mime;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models.Interfaces;

namespace ODM.Backend.Models.Models
{
	public class Meal : IMeal, IDbItem
	{
		public int Id { get; set; }
		public bool IsRemoved { get; set; }
		public int MenuId { get; set; }

		public string Text { get; set; }
		public MealType Type { get; set; }

		public DateTime Date { get; set; }
	}

	public sealed class MealComparer : IEqualityComparer<Meal>
	{
		public bool Equals(Meal x, Meal y)
		{
			if (x == null)
				return y == null;
			else if (y == null)
				return false;
			else
				return x.Id.Equals(y.Id)
				       && x.IsRemoved.Equals(y.IsRemoved)
				       && x.Type.Equals(y.Type)
				       && x.MenuId.Equals(y.MenuId)
				       && x.Date.Equals(y.Date)
				       && x.Text.Equals(y.Text);
		}

		public int GetHashCode(Meal obj)
		{
			unchecked
			{
				int hash = 17;

				hash = hash * 23 + (obj.Id).GetHashCode();
				hash = hash * 23 + (obj.IsRemoved).GetHashCode();
				hash = hash * 23 + (obj.Type).GetHashCode();
				hash = hash * 23 + (obj.Date).GetHashCode();
				hash = hash * 23 + (obj.Text ?? "").GetHashCode();
				hash = hash * 23 + (obj.MenuId).GetHashCode();

				return hash;
			}
		}
	}
}