﻿using System;
using System.Collections;
using System.Collections.Generic;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models.Interfaces;

namespace ODM.Backend.Models.Models
{
	public class Message : IMessage, IDbItem
	{
		public int Id { get; set; }
		public bool IsRemoved { get; set; }
		public string Title { get; set; }
		public bool IsImportant { get; set; }
		public MessageTopicEnum Topic { get; set; }
		public DateTime Date { get; set; }
		public string Text { get; set; }
	}

	public sealed class MessageComparer : IEqualityComparer<Message>
	{
		public bool Equals(Message x, Message y)
		{
			if (x == null)
				return y == null;
			else if (y == null)
				return false;
			else
				return x.Id.Equals(y.Id)
					   && x.IsRemoved.Equals(y.IsRemoved)
					   && x.Title.Equals(y.Title)
					   && x.IsImportant.Equals(y.IsImportant)
					   && x.Topic.Equals(y.Topic)
					   && x.Date.Equals(y.Date)
					   && x.Text.Equals(y.Text);
		}

		public int GetHashCode(Message obj)
		{
			unchecked
			{
				int hash = 17;

				hash = hash * 23 + (obj.Id).GetHashCode();
				hash = hash * 23 + (obj.IsRemoved).GetHashCode();
				hash = hash * 23 + (obj.Title ?? "").GetHashCode();
				hash = hash * 23 + (obj.IsImportant).GetHashCode();
				hash = hash * 23 + (obj.Topic).GetHashCode();
				hash = hash * 23 + (obj.Date).GetHashCode();
				hash = hash * 23 + (obj.Text ?? "").GetHashCode();

				return hash;
			}
		}
	}
}