﻿using System.Collections.Generic;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models.Interfaces;

namespace ODM.Backend.Models.Models
{
	public class Venue : IVenue, IDbItem
	{
		public int Id { get; set; }
		public bool IsRemoved { get; set; }

		public string Name { get; set; }

		public VenueTypeEnum Type { get; set; }

		public Gps Location { get; set; }

		public Address Address { get; set; }

		public Venue(int id, string name, VenueTypeEnum type, double latitude, double longitude, string street, string streetNumber, string city)
		{
			Id = id;
			Name = name;
			Type = type;
			Location = new Gps(latitude, longitude);
			Address = new Address(street, streetNumber, city);
		}

		public Venue()
		{

		}
	}


	public sealed class VenueComparer : IEqualityComparer<Venue>
	{
		public bool Equals(Venue x, Venue y)
		{
			if (x == null)
				return y == null;
			else if (y == null)
				return false;
			else
				return x.Id.Equals(y.Id)
					   && x.IsRemoved.Equals(y.IsRemoved)
					   && x.Type.Equals(y.Type)
					   && x.Location.Longitude.Equals(y.Location.Longitude)
					   && x.Location.Latitude.Equals(y.Location.Latitude)
					   && x.Name.Equals(y.Name)
					   && x.Address.City.Equals(y.Address.City)
					   && x.Address.StreetNumber.Equals(y.Address.StreetNumber)
					   && x.Address.Street.Equals(y.Address.Street);
		}

		public int GetHashCode(Venue obj)
		{
			unchecked
			{
				int hash = 17;

				hash = hash * 23 + (obj.Id).GetHashCode();
				hash = hash * 23 + (obj.IsRemoved).GetHashCode();
				hash = hash * 23 + (obj.Type).GetHashCode();
				hash = hash * 23 + (obj.Location.Longitude).GetHashCode();
				hash = hash * 23 + (obj.Location.Latitude).GetHashCode();
				hash = hash * 23 + (obj.Name ?? "").GetHashCode();
				hash = hash * 23 + (obj.Address.City).GetHashCode();
				hash = hash * 23 + (obj.Address.StreetNumber).GetHashCode();
				hash = hash * 23 + (obj.Address.Street).GetHashCode();

				return hash;
			}
		}
	}
}