﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using ODM.Backend.Models.Models.Interfaces;
using ODMWebService.DL.Entities.Enums;
using Skeleton.Mobile.Framework.Data.Manager;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class CompetitionManager : AbsManager
	{
		/// <summary>
		/// Vrátí seznam všech soutěží
		/// </summary>
		/// <returns></returns>
		public async Task<List<Competition>> GetAll()
		{
			try
			{
				var result = (await DataManager.GetProgramData()).Competitions;

				return result;
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}

		public async Task<List<Competition>> GetByVenue(Venue venue)
		{
			try
			{
				var comps = await this.GetAll();
				var output = new List<Competition>();
				var eventManager = new EventManager();

				foreach (var competition in comps)
				{
					var events = await eventManager.GetEventsByCompetition(competition);
					bool contains = false; //TODO: zjistit co jsem to tu provedl za sračku, nevim proč to tu je

					if (!contains)
					{
						foreach (var @event in events)
						{//TODO: opravit ten loop ať nemusim používat distinct
							contains = @event.VenueId == venue.Id;
							if (contains) output.Add(competition);
						}
					}

				}

				output = output.Distinct().ToList();
				return output;
			}
			catch (Exception e)
			{
				await this.HandleException(e);
				return null;
			}
		}

		public async Task<List<Competition>> GetByCity(string city)
		{

			try
			{
				var comps = await this.GetAll();
				var output = new List<Competition>();
				var eventManager = new EventManager();

				foreach(var comp in comps){
					if (comp.Cities.Contains(city))
					{
						output.Add(comp);
					}
				}
				return output;
			}
			catch (Exception e)
			{
				await this.HandleException(e);
				return null;
			}

		}

		public async Task<List<Competition>> GetBySport(Sport sport)
		{
			try
			{
				var all = await this.GetAll();
				var competitions = all?.Where(i => i.SportId == sport.Id).ToList();
				return competitions;
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}

		public async Task<Competition> GetSideProgram(CompetitionTypeEnum compType)
		{
			try
			{
				var all = await this.GetAll();
				var competition = all?.First(c => c.CompetitionType == compType);
				return competition;
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}
	}
}