﻿using System;
using System.IO;
using System.Text;
using System.Threading.Tasks;
using ODM.Backend.Models.XmlSettings;
using Skeleton.Mobile.Framework.Data.Manager;
using Skeleton.Mobile.Framework.Device;
using Skeleton.Mobile.ODM.Backend.DataAccess.WebService;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class DataUpdateManager
	{
		private DataUpdateManager()
		{

		}
		private static DataUpdateManager instance = null;
		public static DataUpdateManager Instance
		{
			get
			{
				if (instance == null)
				{
					instance = new DataUpdateManager();
				}
				return instance;
			}
		}
		public async Task CheckOrUpdateData()
		{
			var state = await this.GetFileState();
			if (state.LastStaticFileVersion != 0)
			{
				await this.UpdateStaticData(state.LastStaticFileVersion);
			}
			if (state.LastMessageFileVersion != 0)
			{
				await this.UpdateMessageData(state.LastMessageFileVersion);
			}
			if (state.LastProgramFileVersion != 0)
			{
				await this.UpdateProgramData(state.LastProgramFileVersion);
			}
			if (state.LastResultFileVersion != 0)
			{
				await this.UpdateResultsData(state.LastResultFileVersion);
			}
		}

		public async Task<bool> CheckOrUpdateMessages()
		{
			var state = await this.GetFileState();
			bool result = false;
			if (state.LastMessageFileVersion != 0)
			{
				result = await this.UpdateMessageData(state.LastMessageFileVersion);
			}

			return result;
		}

		/// <summary>
		/// Zjistí, zda je potřeba znovu stáhnout data a pokud ano, učiní tak
		/// </summary>
		/// <returns>Was updated true / false</returns>
		internal async Task<bool> UpdateStaticData(int version)
		{
			var repository = new ODMWebRepository();
			int localVersion = SettingsManager.Instance.StaticVersion;
			var needsUpdate = localVersion < version;
			if (needsUpdate)
			{
				var data = await repository.GetStaticData(localVersion);

				if (data != null)
				{
					var dataVersion = await DataManager.UpdateStaticData(data);
					if (version != 0)
					{
						SettingsManager.Instance.StaticVersion = dataVersion;
						return true;
					}
					return false;
				}
			}
			return false;
		}

		/// <summary>
		/// Zjistí, zda je potřeba znovu stáhnout data a pokud ano, učiní tak
		/// </summary>
		/// <returns>Was updated true / false</returns>
		internal async Task<bool> UpdateMessageData(int version)
		{
			var repository = new ODMWebRepository();
			int localVersion = SettingsManager.Instance.MessageVersion;
			var needsUpdate = localVersion < version;
			if (needsUpdate)
			{
				var data = await repository.GetMessageData(localVersion);

				if (data != null)
				{
					var dataVersion = await DataManager.UpdateMessageData(data);
					if (version != 0)
					{
						SettingsManager.Instance.MessageVersion = dataVersion;
						return true;
					}
					return false;
				}
			}
			return false;
		}


		/// <summary>
		/// Zjistí, zda je potřeba znovu stáhnout data a pokud ano, učiní tak
		/// </summary>
		/// <returns>Was updated true / false</returns>
		internal async Task<bool> UpdateProgramData(int version)
		{
			var repository = new ODMWebRepository();
			int localVersion = SettingsManager.Instance.ProgramVersion;
			var needsUpdate = localVersion < version;
			if (needsUpdate)
			{
				var data = await repository.GetProgramData(localVersion);

				if (data != null)
				{
					var dataVersion = await DataManager.UpdateProgramData(data);
					if (version != 0)
					{
						SettingsManager.Instance.ProgramVersion = dataVersion;
						return true;
					}
					return false;
				}
			}
			return false;
		}

		/// <summary>
		/// Zjistí, zda je potřeba znovu stáhnout data a pokud ano, učiní tak
		/// </summary>
		/// <returns>Was updated true / false</returns>
		internal async Task<bool> UpdateResultsData(int version)
		{
			var repository = new ODMWebRepository();
			int localVersion = SettingsManager.Instance.ResultsVersion;
			var needsUpdate = localVersion < version;
			if (needsUpdate)
			{
				var data = await repository.GetResultsData(localVersion);

				if (data != null)
				{
					var dataVersion = await DataManager.UpdateResultsData(data);
					if (version != 0)
					{
						SettingsManager.Instance.ResultsVersion = dataVersion;
						return true;
					}
					return false;
				}
			}

			return false;
		}

		

		internal async Task<FileState> GetFileState()
		{
			var repository = new ODMWebRepository();
			var data = await repository.CheckDataState();
			return data;
		}
	}
}