﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Data.Manager;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class EventManager : AbsManager
	{
		/// <summary>
		/// Vrátí detail požadovaného místa
		/// </summary>
		/// <param name="id">id místa</param>
		/// <returns></returns>
		public async Task<List<Event>> GetEventsByCompetition(Competition comp)
		{
			try
			{
				var output = await GetAll();
				return output.Where(e => e.CompetitionId == comp.Id).ToList();
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}

		public async Task<List<Event>> GetEventsByCity(string city)
		{
			try
			{
				List<Event> output = new List<Event>();
				var venuemanager = new VenueManager();
				var venues = await venuemanager.GetByCity(city);
				
				foreach(var venue in venues)
				{
					var range = await GetEventsByVenueId(venue.Id);
					output.AddRange(range);
				}
				return output;
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}

		public async Task<List<Event>> GetEventsByVenueId(int id)
		{
			try
			{
				var output = await GetAll();
				return output.Where(e => e.VenueId == id).ToList();
			}
			catch (Exception e)
			{
				await this.HandleException(e);
				return null;
			}
		}

		/// <summary>
		/// Vrátí seznam všech kolektivních soutěží
		/// </summary>
		/// <returns></returns>
		public async Task<List<Event>> GetAll()
		{
			try
			{
				var result = (await DataManager.GetProgramData()).Events;
				return result;
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}
	}
}