﻿using System;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Skeleton.Mobile.Framework.Data.Localization;
using Skeleton.Mobile.Framework.Data.Manager;
using Skeleton.Mobile.Framework.Device;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.Framework.Error.Log;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class LocationManager : AbsManager
	{
		private LocationManager()
		{

		}

		private static LocationManager instance;

		public static LocationManager Instance
		{
			get
			{
				if (instance == null)
				{
					instance = new LocationManager();
				}

				return instance;
			}
		}

		private Gps currentLocation;

		public string GetAddress()
		{
			return SettingsManager.Instance.HousingAddress;
		}

		private async Task<Gps> StartSearchingLocation()
		{
			this.currentLocation = null;

			var perm = await Device.Instance.PermissionsManager.RequestLocationInUsePermission();

			if (perm)
			{

				await
					ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Network enabled - {Framework.Device.Device.Instance.LocationManager.NetworkLocationEnabled}",
						"VenueListViewModel - searching location", false);

				return await Device.Instance.LocationManager.GetCurrentLocation(new CancellationToken(), 10000);
			}

			return null;
		}

		private Gps GetHousingLocationFromSettings()
		{
			return SettingsManager.Instance.HousingLocation;
		}

		public bool GetLocationWasSet => SettingsManager.Instance.IsHousingSet;
		//TODO: geocoder ve frameworku je rozbitej

		public async Task<Gps> SetHousingGps(){
			Gps loc = null;
			try
			{
				loc = await StartSearchingLocation();
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}

			if(loc!= null){
				SettingsManager.Instance.HousingLocation = loc;
			}
			return loc;
		}

		public async Task<bool> SetHousingLocation(Gps gps)
		{
			var loc = gps;

			if (loc != null)
			{
				Address address = null;
				try
				{
					address = await Device.Instance.GeocoderManager.GetAddressesForPosition(loc);
				}
				catch (Exception ex)
				{
					await this.HandleException(ex);
					return false;
				}

				SettingsManager.Instance.HousingAddress = address.MultiLineAddress;

				return true;
			}
			return false;
		}

		public Gps GetHousingLocation()
		{
			return GetHousingLocationFromSettings();
		}
	}
}