﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Data.Manager;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class MyCalendarManager : AbsManager
	{
		private MyCalendarManager()
		{
			
		}
		private static MyCalendarManager instance;
		public static MyCalendarManager Instance
		{
			get
			{
				if (instance == null)
				{
					instance = new MyCalendarManager();
				}
				return instance;
			}
		}

		private static List<int> CalendarEvents { get; set; }

		public void AddEventToCalendar(Event eventModel)
		{
			if (CalendarEvents == null)
			{
				CalendarEvents = SettingsManager.Instance.MyCalendarEvents ?? new List<int>();
			}

			if (!CalendarEvents.Contains(eventModel.Id))
			{
				CalendarEvents.Add(eventModel.Id);
				Task.Run(() => { SettingsManager.Instance.MyCalendarEvents = CalendarEvents; });
			}
		}

		public void RemoveEventFromCalendar(Event eventModel)
		{
			if (CalendarEvents == null)
			{
				CalendarEvents = SettingsManager.Instance.MyCalendarEvents ?? new List<int>();
			}

			if (CalendarEvents.Contains(eventModel.Id))
			{
				CalendarEvents.Remove(eventModel.Id);
				Task.Run(() => { SettingsManager.Instance.MyCalendarEvents = CalendarEvents; });
			}
		}

		public bool CheckIfEventIsInCalendar(Event eventModel)
		{
			if (CalendarEvents == null)
			{
				CalendarEvents = SettingsManager.Instance.MyCalendarEvents ?? new List<int>();
			}
			return CalendarEvents.Contains(eventModel.Id);
		}

		public List<int> GetAllIds()
		{
			if (CalendarEvents == null)
			{
				CalendarEvents = SettingsManager.Instance.MyCalendarEvents ?? new List<int>();
			}

			return CalendarEvents;
		}

		public async Task<List<Event>> GetCalendarEvents()
		{
			List<Event> output = new List<Event>();

			if (CalendarEvents == null)
			{
				CalendarEvents = SettingsManager.Instance.MyCalendarEvents ?? new List<int>();
			}

			var eventManager = new EventManager();

			List<Event> events;
			try
			{
				events = await eventManager.GetAll();
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				events = new List<Event>();
			}

			foreach (var id in CalendarEvents)
			{
				var ev = events.Find(e => e.Id == id);
				if (ev != null) output.Add(ev);
			}

			return output;
		}

	}
}