﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Newtonsoft.Json;
using Skeleton.Mobile.Framework.Device;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.ODM.Backend.Models;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class SettingsManager
	{
		private SettingsManager()
		{

		}

		private static SettingsManager instance = null;

		public static SettingsManager Instance
		{
			get
			{
				if (instance == null)
				{
					instance = new SettingsManager();
				}

				return instance;
			}
		}

		private string StaticUpdateKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.StaticUpdateKey";
		private string MessageUpdateKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.MessageUpdateKey";
		private string ProgramUpdateKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.ProgramUpdateKey";
		private string ResultsUpdateKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.ResultsUpdateKey";

		private string HousingIsSetKey => "Skeleton.Mobile.Odm.Backend.Managers.SettingsManager.HousingIsSetKey";
		private string HousingLatitudeKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.HousingLatitudeKey";
		private string HousingLongitudeKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.HousingLongitudeKey";
		private string HousingAddressKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.HousingAddressKey";

		private string MyCalendarKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.MyCalendarKey";
		private string MyNotificationsKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.MyNotificationsKey";
		private string NotificationIdPointerKey => "Skeleton.Mobile.ODM.Backend.Managers.SettingsManager.NotificationIdPointerKey";

		internal List<int> MyCalendarEvents
		{
			get
			{
				string myCalJson = Device.Instance.Settings.GetVariable(MyCalendarKey, "");

				if (myCalJson == "")
				{
					MyCalendarEvents = new List<int>();
					myCalJson = Device.Instance.Settings.GetVariable(MyCalendarKey, "");
				}

				return JsonConvert.DeserializeObject<List<int>>(myCalJson);
			}
			set
			{
				string myNotificationsJson = JsonConvert.SerializeObject(value);
				Device.Instance.Settings.SetVariable(myNotificationsJson, MyCalendarKey);
			}
		}

		internal static class Notifications
		{
			public static int IncrementNotificationId(){
				int lastId = Instance.NotificationIdPointer;
				return lastId;
			}

			public static List<Notification> GetAllNotifications()
			{
				return SettingsManager.Instance.MyNotifications;
			}

			public static void AddNotification(Notification notification)
			{
				List<Notification> notifs = SettingsManager.Instance.MyNotifications;
				notification.NotificationId = Instance.NotificationIdPointer;
				notifs.Add(notification);
				Instance.MyNotifications = notifs;
			}

			public static void RemoveNotification(Notification notification)
			{
				RemoveNotificationById(notification.NotificationId);
			}

			public static void RemoveNotificationById(int id)
			{
				List<Notification> notifs = SettingsManager.Instance.MyNotifications;
				var notif = notifs.Find(n => n.NotificationId == id);
				notifs.Remove(notif);
				Instance.MyNotifications = notifs;
			}

			public static void Clear(bool uSureQuestionMark = false)
			{
				if (uSureQuestionMark)
				{
					Instance.MyNotifications = new List<Notification>();
				}
			}
		}
		private int NotificationIdPointer {
			get{
				int id = Device.Instance.Settings.GetVariable(NotificationIdPointerKey, 0);
				id++;
				Device.Instance.Settings.SetVariable(id, NotificationIdPointerKey);
				return id;
			}
		}

		private List<Notification> MyNotifications
		{
			get
			{
				string myNotifs = Device.Instance.Settings.GetVariable(MyNotificationsKey, "");

				if (myNotifs == "")
				{
					MyNotifications = new List<Notification>();
					myNotifs = Device.Instance.Settings.GetVariable(MyNotificationsKey, "");
				}

				return JsonConvert.DeserializeObject<List<Notification>>(myNotifs);
			}
			set
			{
				string myNotificationsJson = JsonConvert.SerializeObject(value);
				Device.Instance.Settings.SetVariable(myNotificationsJson, MyNotificationsKey);
			}
		}

		#region Housing

		internal bool IsHousingSet
		{
			get => Device.Instance.Settings.GetVariable(HousingIsSetKey, false);
			set => Device.Instance.Settings.SetVariable(value, HousingIsSetKey);
		}

		internal string HousingAddress
		{
			get { return Device.Instance.Settings.GetVariable(HousingAddressKey, ""); }
			set { Device.Instance.Settings.SetVariable(value, HousingAddressKey); }
		}

		internal Gps HousingLocation
		{
			get
			{
				if (IsHousingSet)
				{
					string lon = Device.Instance.Settings.GetVariable(HousingLongitudeKey, "");
					string lat = Device.Instance.Settings.GetVariable(HousingLatitudeKey, "");

					if (lat != "" && lon != "")
					{
						return new Gps()
						{
							Longitude = float.Parse(lon),
							Latitude = float.Parse(lat)
						};
					}
				}
				return null;
			}

			set
			{
				double lat = value.Latitude;
				double lon = value.Longitude;

				Device.Instance.Settings.SetVariable(lat.ToString(), HousingLatitudeKey);
				Device.Instance.Settings.SetVariable(lon.ToString(), HousingLongitudeKey);
				IsHousingSet = true;
			}
		}
		#endregion

		#region Data Version

		public int StaticVersion
		{
			get { return Device.Instance.Settings.GetVariable(StaticUpdateKey, 0); }
			internal set
			{
				Device.Instance.Settings.SetVariable(value, StaticUpdateKey);
			}
		}

		public int MessageVersion
		{
			get { return Device.Instance.Settings.GetVariable(MessageUpdateKey, 0); }
			internal set
			{
				Device.Instance.Settings.SetVariable(value, MessageUpdateKey);
			}
		}

		public int ProgramVersion
		{
			get { return Device.Instance.Settings.GetVariable(ProgramUpdateKey, 0); }
			internal set
			{
				Device.Instance.Settings.SetVariable(value, ProgramUpdateKey);
			}
		}

		public int ResultsVersion
		{
			get { return Device.Instance.Settings.GetVariable(ResultsUpdateKey, 0); }
			internal set
			{
				Device.Instance.Settings.SetVariable(value, ResultsUpdateKey);
			}
		}
		#endregion

	}
}