﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Data.Manager;
using System.Linq;
using System.Linq.Expressions;
using ODM.Backend.Models.Enums;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	public class VenueManager : AbsManager
	{
		/// <summary>
		/// Vrátí detail požadovaného místa
		/// </summary>
		/// <param name="id">id místa</param>
		/// <returns></returns>
		public async Task<Venue> GetById(int id)
		{
			try
			{
				var output = await GetAll();
				return output.Single(x => x.Id == id);
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}

		/// <summary>
		/// Vrátí seznam všech venues
		/// </summary>
		/// <returns></returns>
		public async Task<List<Venue>> GetAll()
		{
			try
			{
				var result = (await DataManager.GetStaticData()).Venues;

				return result.Where(x => x.Location != null).OrderBy(x => x.Name).ToList();
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}
		/// <summary>
		/// Vrátí místo podle města, ve kterém se nachází
		/// get cities from GetCities
		/// </summary>
		public async Task<List<Venue>> GetByCity(string city)
		{
			var vens = await this.GetAll();
			return vens?.Where(v => v.Address.City == city).ToList();
		}

		/// <summary>
		/// Vrátí list měst
		/// </summary>
		public async Task<List<string>> GetCities()
		{
			try
			{
				var result = (await DataManager.GetStaticData()).Cities;
				return result;
			}
			catch (Exception ex)
			{
				await this.HandleException(ex);
				return null;
			}
		}

		public async Task<List<Venue>> GetByType(VenueTypeEnum enu)
		{
			var vens = await this.GetAll();
			return vens.Where(v => v.Type == enu).ToList();
		}

		public async Task<List<Venue>> GetBySport(Sport sport)
		{
			var vens = await this.GetAll();

			var compMan = new CompetitionManager();
			var eventMan = new EventManager();

			List<int> venueIds = new List<int>();

			var competitions = await compMan.GetBySport(sport);

			List<Event> events = new List<Event>();

			if (competitions != null)
			{
				foreach (var comp in competitions)
				{
					//venueIds.Add(comp.VenueId);
					var compEvents = await eventMan.GetEventsByCompetition(comp);

					events.AddRange(compEvents);
				}
			}
			//filtrace duplikátů
			events = events.Distinct().ToList();

			foreach (var @event in events)
			{
				venueIds.Add(@event.VenueId);
			}

			var output = new List<Venue>();

			foreach (var id in venueIds)
			{
				var venue = await GetById(id);

				if (!output.Contains(venue))
				{
					output.Add(venue);
				}
			}

			return output;
		}

	}
}