﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Android.App;
using Android.Content;
using Android.Graphics;
using Android.Media;
using Android.OS;
using Android.Runtime;
using Android.Support.V4.App;
using Android.Views;
using Android.Widget;
using Skeleton.Mobile.ODM.Tools;

namespace Skeleton.Mobile.ODM.Droid.Notifications
{
	[BroadcastReceiver(Enabled = true, Label = "Notifications Broadcast Receiver")]
	public class AlarmBroadcastReceiver : BroadcastReceiver
	{
		//přišel požadavek na zobrazení notifikace - vyrobíme a ihned zobrazíme nativní lokální notifikaci
		public override void OnReceive(Context context, Intent intent)
		{
			CreateAndShowNotification(intent);
		}

		private void CreateAndShowNotification(Intent intent)
		{
			int notificationId = intent.NotificationId();
			int eventId = intent.EventId();
			int eventType = intent.EventType();
			string title = intent.Title();
			string message = intent.Message();
			bool vibrate = intent.Vibrate();
			string channelId = intent.ChannelId();

			var launchIntent = Application.Context.PackageManager.GetLaunchIntentForPackage(Application.Context.PackageName);
			launchIntent.SetFlags(ActivityFlags.NewTask | ActivityFlags.ClearTask);

			launchIntent.PutExtra(NotificationFactory.EventIdKey, eventId);

			Notification.Builder notificationBuilder;

			if (Build.VERSION.SdkInt < BuildVersionCodes.O)
			{
#pragma warning disable CS0618 // Type or member is obsolete but used only in older SDKs, so its ok...
				notificationBuilder = new Notification.Builder(Application.Context)
					.SetDefaults(NotificationDefaults.All)
					.SetPriority((int)NotificationPriority.Default)
					.SetSound(RingtoneManager.GetDefaultUri(RingtoneType.Notification));

				if (vibrate)
				{
					notificationBuilder.SetVibrate(new long[] { 500, 500, 500, 500 });
				}
#pragma warning restore CS0618 // Type or member is obsolete
			}
			else
			{
				// od Android O se používají channels, uživatel si sám nastaví prioritu a vlastnosti notifikací daného kanálu
				notificationBuilder = new Notification.Builder(Application.Context, channelId);
			}

			notificationBuilder
				.SetSmallIcon(Resource.Mipmap.ic_launcher)
				.SetContentTitle(title)
				.SetContentText(message)
				.SetAutoCancel(true)
				.SetContentIntent(Android.App.TaskStackBuilder
					.Create(Application.Context)
					.AddNextIntent(launchIntent)
					.GetPendingIntent(notificationId, PendingIntentFlags.OneShot)
				);

			var not = notificationBuilder.Build();
			NotificationManagerCompat.From(Application.Context).Notify(notificationId, not);
		}
	}
}