﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Android.App;
using Android.Content;
using Android.OS;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Notification = Skeleton.Mobile.ODM.Backend.Models.Notification;

namespace Skeleton.Mobile.ODM.Droid.Notifications
{
	public static class Helpers
	{
		private const string NotificationIdKey = "id";
		private const string EventIdKey = "eventId";
		private const string EventTypeKey = "eventType";
		private const string TitleKey = "title";
		private const string MessageKey = "message";
		private const string VibrateKey = "vibrate";
		private const string ChannelIdKey = "channelId";

		public static PendingIntent ToPendingIntent(int id, int eventId, string title, string message, bool vibrate, string channelId)
		{
			var intent = new Intent(Application.Context, typeof(AlarmBroadcastReceiver)).PutExtra(NotificationIdKey, id);
			intent.PutExtra(EventIdKey, eventId);
			intent.PutExtra(TitleKey, title);
			intent.PutExtra(MessageKey, message);
			intent.PutExtra(VibrateKey, vibrate);
			intent.PutExtra(ChannelIdKey, channelId);
			var pending = PendingIntent.GetBroadcast(Application.Context, id, intent, PendingIntentFlags.OneShot);
			return pending;
		}

		public static PendingIntent GetNotificationPendingIntent(int id)
		{
			var intent = new Intent(Application.Context, typeof(AlarmBroadcastReceiver)).PutExtra(NotificationIdKey, id);
			var pending = PendingIntent.GetBroadcast(Application.Context, id, intent, PendingIntentFlags.OneShot);
			return pending;
		}

		public static int NotificationId(this Intent intent) => intent.GetIntExtra(NotificationIdKey, -1);
		public static int EventId(this Intent intent) => intent.GetIntExtra(EventIdKey, -1);
		public static int EventType(this Intent intent) => intent.GetIntExtra(EventTypeKey, 0);
		public static string Title(this Intent intent) => intent.GetStringExtra(TitleKey);
		public static string Message(this Intent intent) => intent.GetStringExtra(MessageKey);
		public static bool Vibrate(this Intent intent) => intent.GetBooleanExtra(VibrateKey, false);
		public static string ChannelId(this Intent intent) => intent.GetStringExtra(ChannelIdKey);

		public static long GetEpochMills(DateTime sendTime)
		{
			var utc = sendTime.ToUniversalTime();
			var epochDiff = (new DateTime(1970, 1, 1) - DateTime.MinValue).TotalSeconds;
			var utcAlarmTimeInMillis = utc.AddSeconds(-epochDiff).Ticks / 10000;
			return utcAlarmTimeInMillis;
		}

		public static Task ScheduleNotification(this Android.App.AlarmManager alarmManager, Notification notification)
		{
			var triggerMs = Helpers.GetEpochMills(notification.FireDateTime);
			var pending = Helpers.ToPendingIntent(notification.NotificationId, notification.EventId, notification.Title, notification.Message, notification.Vibrate, notification.ChannelId);

			if (Build.VERSION.SdkInt >= BuildVersionCodes.M)
			{
				// SetExactAndAllowWhileIdle přidáno až v API 23
				// použito, protože funguje v Doze mode a zároveň v Oreo po omezení background services
				// vhodné na kalendářová upozornění
				// https://developer.android.com/reference/android/app/AlarmManager#setExactAndAllowWhileIdle(int,%2520long,%2520android.app.PendingIntent)
				alarmManager.SetExactAndAllowWhileIdle(Android.App.AlarmType.RtcWakeup, Convert.ToInt64(triggerMs), pending);
			}
			else
			{
				alarmManager.Set(Android.App.AlarmType.RtcWakeup, Convert.ToInt64(triggerMs), pending);
			}

			return Task.CompletedTask;
		}
	}
}