﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Foundation;
using Skeleton.Mobile.Framework.Tools.iOS;
using Skeleton.Mobile.ODM.Backend.Models;
using Skeleton.Mobile.ODM.Tools;
using UIKit;

namespace Skeleton.Mobile.ODM.iOS.Notifications
{
	public class UILocalNotificationsImpl : AbstractAppleNotificationsImpl
	{
		const string NOTIFICATION_ID_KEY = "NotificationID";

		public override Task CancelByNotificationId(int notificationId)
		{
			var key = new NSString(NOTIFICATION_ID_KEY);
			var keyValue = new NSNumber(notificationId);

			var notification = UIApplication.SharedApplication.ScheduledLocalNotifications.FirstOrDefault(x =>
				x.UserInfo.ContainsKey(key) &&
				x.UserInfo[key].Equals(keyValue)
			);
			if (notification == null)
				return Task.CompletedTask;

			return this.Invoke(() =>
				UIApplication.SharedApplication.CancelLocalNotification(notification)
			);
		}

		public override Task Send(Notification notification)
		{
			NSMutableDictionary userInfo = this.CreateUserInfoDictionary(notification);
			userInfo.Add(NSObject.FromObject(NOTIFICATION_ID_KEY), NSObject.FromObject(notification.NotificationId)); //přidání id notifikace

			var not = new UILocalNotification();
			not.FireDate = notification.FireDateTime.ToNSDate();
			not.AlertTitle = notification.Title;
			not.AlertBody = notification.Message;
			not.SoundName = notification.Sound;
			not.UserInfo = userInfo;
			return this.Invoke(() => UIApplication.SharedApplication.ScheduleLocalNotification(not));
		}

		public override Task<bool> RequestPermission()
		{
			var settings = UIUserNotificationSettings.GetSettingsForTypes(
				UIUserNotificationType.Alert | UIUserNotificationType.Badge | UIUserNotificationType.Sound,
				null
			);
			UIApplication.SharedApplication.RegisterUserNotificationSettings(settings);
			return Task.FromResult(true);
		}
	}
}