﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Foundation;
using Skeleton.Mobile.Framework.Tools.iOS;
using Skeleton.Mobile.ODM.Backend.Models;
using Skeleton.Mobile.ODM.Tools;
using UIKit;
using UserNotifications;

namespace Skeleton.Mobile.ODM.iOS.Notifications
{
	public class UNNotificationsImpl : AbstractAppleNotificationsImpl
	{
		public override Task CancelByNotificationId(int notificationId) => this.Invoke(() =>
		{
			var ids = new[] { notificationId.ToString() };
			UNUserNotificationCenter.Current.RemovePendingNotificationRequests(ids);
			UNUserNotificationCenter.Current.RemoveDeliveredNotifications(ids);
		});

		public override Task Send(Notification notification) => this.Invoke(async () =>
		{
			var content = new UNMutableNotificationContent
			{
				Title = notification.Title,
				Body = notification.Message
			};
			content.UserInfo = this.CreateUserInfoDictionary(notification);

			if (!String.IsNullOrWhiteSpace(notification.Sound))
				content.Sound = UNNotificationSound.GetSound(notification.Sound);

			var dt = notification.FireDateTime;
			var request = UNNotificationRequest.FromIdentifier(
				notification.NotificationId.ToString(),
				content,
				UNCalendarNotificationTrigger.CreateTrigger(new NSDateComponents
				{
					Year = dt.Year,
					Month = dt.Month,
					Day = dt.Day,
					Hour = dt.Hour,
					Minute = dt.Minute,
					Second = dt.Second
				}, false)
			);
			await UNUserNotificationCenter.Current.AddNotificationRequestAsync(request);
		});

		public override Task<bool> RequestPermission()
		{
			var tcs = new TaskCompletionSource<bool>();
			UNUserNotificationCenter.Current.RequestAuthorization(
				UNAuthorizationOptions.Alert | UNAuthorizationOptions.Badge | UNAuthorizationOptions.Sound,
				(approved, error) => tcs.TrySetResult(approved)
			);
			return tcs.Task;
		}
	}

}