﻿using ODM.Backend.Models.Models;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Backend.Models;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;

namespace Skeleton.Mobile.ODM.Tools
{
	public static class NotificationFactory
	{
		public const string EventIdKey = "EventId";

		public const string EventsChannelId = "ODM.EventsChannel";
		
		internal static async Task RefreshNotifications()
		{
			await CrossNotifications.Current.RequestPermission();
			await CancelAll();
			int? minutesBeforeEvent = GetMinutesBeforeEvent();

			var eventManager = MyCalendarManager.Instance;
			List<Event> allCalendarEvents;
			using (var managerassistant = new ManagerAssistant(null, eventManager, string.Empty, false))
			{
				allCalendarEvents = await eventManager.GetCalendarEvents();
			}
			
			
			if (minutesBeforeEvent.HasValue)
			{
				foreach (var item in allCalendarEvents)
				{
					if (item.Start > DateTime.Now)
					{
						await CreateNotification(item, minutesBeforeEvent);
					}
				}
			}
		}

		public static async Task DeleteNotificationById(int notificationId, bool onlyFromDb = false)
		{
			await CrossNotifications.Current.RequestPermission();

			NotificationManager manager = new NotificationManager();
			using (var assistant = new ManagerAssistant(null, manager, string.Empty, false))
			{
				//zda chceme záznam smazat pouze z databáze (míň záznamů - zefektivnění práce), nebo notifikaci zrušit i v telefonu (pak se ale zobrazená notifikace schová)
				if (!onlyFromDb)
				{
					await CrossNotifications.Current.CancelByNotificationId(notificationId);
				}
				manager.RemoveNotificationById(notificationId);
			}
		}

		public static async Task DeleteNotificationByEventId(int eventId, bool onlyFromDb = false)
		{
			await CrossNotifications.Current.RequestPermission();

			NotificationManager manager = new NotificationManager();
			using (var assistant = new ManagerAssistant(null, manager, string.Empty, false))
			{
				var notification = manager.GetNotificationByEventId(eventId);
				//zda chceme záznam smazat pouze z databáze (míň záznamů - zefektivnění práce), nebo notifikaci zrušit i v telefonu (pak se ale zobrazená notifikace schová)
				if (!onlyFromDb && notification != null)
				{
					await CrossNotifications.Current.CancelByNotificationId(notification.NotificationId);
				}
				if(notification != null)
				{
					manager.RemoveNotificationById(notification.NotificationId);
				}
			}
		}

		internal static async Task CreateNotification(Event notificationEvent, int? minutesBeforeEvent = null)
		{
			if (!minutesBeforeEvent.HasValue)
				minutesBeforeEvent = GetMinutesBeforeEvent();
			await CrossNotifications.Current.RequestPermission();

			bool future = DateTime.Compare(notificationEvent.Start, DateTime.Now) > 0;

			if (minutesBeforeEvent.HasValue && future)
			{
				//vytvoření notifikací
				DateTime fireDate = notificationEvent.Start.AddMinutes(-minutesBeforeEvent.Value);

				

				Sport eventSport = null;
				Competition comp = null;
				NotificationManager manager = new NotificationManager();
				var sportmanager = new SportManager();
				var competitionManager = new CompetitionManager();
				Notification notification;
				using (var assistant = new ManagerAssistant(null, competitionManager, String.Empty, false))
				{
					comp = (await competitionManager.GetAll()).Find(c => c.Id == notificationEvent.CompetitionId);
					assistant.SetManager(sportmanager);
					eventSport = (await sportmanager.GetAll()).Find(s => s.Id == comp.SportId);

					notification = new Notification()
					{
						EventId = notificationEvent.Id,
						Vibrate = true,
						FireDateTime = fireDate,
						ChannelId = EventsChannelId,

						Title = $"{eventSport?.ToString()} {comp.ToString()} {notificationEvent.Start:dd.MM.yyyy hh:mm}",
						Message = $"{notificationEvent.Title}"
					};

					notification = await manager.AddNotification(notification);
				}


				await CrossNotifications.Current.Send(notification);
			}
		}

		private static async Task CancelAll()
		{
			await CrossNotifications.Current.RequestPermission();

			NotificationManager manager = new NotificationManager();
			using (var assistant = new ManagerAssistant(null, manager, string.Empty, false))
			{
				var allNotifications = manager.GetAllNotifications();

				if (allNotifications != null && allNotifications.Count > 0)
				{
					foreach (var notification in allNotifications)
					{
						await CrossNotifications.Current.CancelByNotificationId(notification.NotificationId);
					}

					manager.RemoveAllNotifications(true);
				}
			}
		}

		private static int? GetMinutesBeforeEvent()
		{
			var output = NotificationManager.MinutesBeforeEvent;
			if (output != -1)
				return output;
			else
				return null;
		}
	}
}
