﻿using System;
using System.Windows.Input;
using Skeleton.Mobile.Framework.Error.Forms.ErrorReport;
using Skeleton.Mobile.ODM.Backend.Configuration;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.Views;
using Skeleton.Mobile.ODM.Views.Abstract;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class AboutAppPageViewModel
	{
		public AboutAppPageViewModel()
		{
			this.SkeletonWebCommand = new Command(this.SkeletonWebCommand_Execute);
			this.VersionTapCommand = new Command(this.VersionTapCommand_Execute);
			this.ErrorButtonCommand = new Command(this.ErrorButtonCommand_Execute);
			this.AppVersion = $"{"AboutApp_Version".LocalizeResx()} {Skeleton.Mobile.Framework.Device.Device.Instance.CurrentAppManager.Version}";
		}

		public string AppVersion { get; set; }

		#region Commands
		public ICommand SkeletonWebCommand { get; private set; }
		public ICommand VersionTapCommand { get; private set; }
		public ICommand ErrorButtonCommand { get; private set; }

		private void SkeletonWebCommand_Execute(object obj)
		{
			Xamarin.Forms.Device.OpenUri(new Uri("AboutApp_SkeletonWebLink".LocalizeResx()));
		}

		private async void VersionTapCommand_Execute(object obj)
		{
#if DEBUG
			ODMPage.SafeGoToPage(new MaintenancePage(new ODMConfigurationFactory()));
#endif

			await Skeleton.Mobile.Framework.Controls.Forms.ViewModels.MaintenancePageViewModel.GoToMaintenancePage(() =>
			{
				ODMPage.SafeGoToPage(new MaintenancePage(new ODMConfigurationFactory()));
			});
		}

		private void ErrorButtonCommand_Execute(object obj)
		{
			INavigation nav;
			if (RootPage.RootNavigationPage != null)
			{
				nav = RootPage.RootNavigationPage.Navigation;
			}
			else
			{
				nav = (App.Current as App).MainPage.Navigation;
			}
			ErrorReportViewModel.ShowPage<ErrorReportPage>(new AssistantFactory(), nav);
		}
		#endregion
	}
}
