﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.Abstract;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Xamarin.Forms;
using System.Linq;
using ODMWebService.DL.Entities.Enums;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class CompetitionDetailViewModel : CompetitionViewModel
	{
		#region Props

		public List<EventItemViewModel> EventViewModels
		{
			get => eventViewModels;
			set { eventViewModels = value; OnPropertyChanged(); }
		}

		public List<DateTime> Dates
		{
			get => dates;
			set { dates = value; OnPropertyChanged(); }
		}

		public EventItemViewModel SelectedEvent
		{
			get { return null; }
			set { selectedEvent = value; }
		}

		#endregion

		#region Fields

		private List<EventItemViewModel> eventViewModelsBackingList;
		private List<EventItemViewModel> eventViewModels;
		private EventItemViewModel selectedEvent;
		private List<DateTime> dates;
		private List<int> calendarEventIds = new List<int>();

		#endregion

		public CompetitionDetailViewModel(Competition comp, Sport sport, bool isOnVenue = false) : base(comp)
		{
			this.Sport = sport;
			this.IsOnVenuePage = isOnVenue;

			ChangeDateCommand = new Command<DateTime>(ChangeDateCommand_Execute);
		}

		public CompetitionDetailViewModel(Competition comp, Sport sport, string city, bool isOnVenue = false) : this(comp, sport, isOnVenue)
		{
			base.city = city;
		}

		public CompetitionDetailViewModel(Competition comp, Sport sport, Venue venue, bool isOnVenue = false) : this(comp, sport, isOnVenue)
		{
			base.Venue = venue;
		}

		public CompetitionDetailViewModel(CompetitionTypeEnum compType, bool isOnVenue = false) : base()
		{
			this.compType = compType; 
			this.Sport = sport;
			this.IsOnVenuePage = isOnVenue;

			ChangeDateCommand = new Command<DateTime>(ChangeDateCommand_Execute);
		}

		private readonly CompetitionTypeEnum compType;

		#region Commands

		public ICommand ChangeDateCommand { get; set; }
		public void ChangeDateCommand_Execute(DateTime buttonVal)
		{
			this.EventViewModels = eventViewModelsBackingList.Where(v => v.Start.Date == buttonVal.Date).ToList();
		}

		#endregion

		public async Task LoadEventsAreInCalendar()
		{
			foreach (var eventItemViewModel in eventViewModelsBackingList)
			{
				var calendarManager = MyCalendarManager.Instance;
				eventItemViewModel.IsInCalendar = calendarManager.CheckIfEventIsInCalendar(eventItemViewModel.eventModel);
			}
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var compManager = new CompetitionManager();
			var eventManager = new EventManager();
			var venueManager = new VenueManager();

			using (var managerAssistant = new ManagerAssistant(this, compManager, "CompetitionDetail_LoadingEvents".LocalizeResx()))
			{
				if (this.Competition == null)
				{
					this.Competition = await compManager.GetSideProgram(this.compType);
				}

				managerAssistant.SetManager(eventManager);
				var filterEvents = await eventManager.GetEventsByCompetition(this.Competition);

				if (Venue != null)
				{
					filterEvents = filterEvents.Where(e => e.VenueId == Venue.Id).ToList();
				}

				if (city != null)
				{
					var vens = await venueManager.GetByCity(city);

					List<Event> filterByCityEvents = new List<Event>();

					foreach (var ven in vens)
					{
						var venueEvents = filterEvents.Where(e => e.VenueId == ven.Id).ToList();

						if (venueEvents.Count != 0)
						{
							filterByCityEvents.AddRange(venueEvents);
						}
					}

					filterEvents = filterByCityEvents;
				}

				List<EventItemViewModel> viewModels = new List<EventItemViewModel>();

				managerAssistant.SetManager(venueManager);

				List<Venue> venues;
				if (Sport != null)
				{
					venues = await venueManager.GetBySport(this.Sport);
				}
				else
				{
					venues = await venueManager.GetAll();
				}

				this.Dates = new List<DateTime>();
				foreach (var unfilteredEvent in filterEvents)
				{
					var eventVenue = venues.First(v => v.Id == unfilteredEvent.VenueId);

					if (!this.Dates.Contains(unfilteredEvent.Start.Date))
					{
						this.Dates.Add(unfilteredEvent.Start.Date);
					}

					var vM = new EventItemViewModel(unfilteredEvent, this.Competition, this.IsOnVenuePage, eventVenue);

					viewModels.Add(vM);
				}
				this.Dates.Sort();
				this.eventViewModelsBackingList = viewModels;
				this.eventViewModelsBackingList.OrderBy(e => e.Start);
				this.EventViewModels = eventViewModelsBackingList.Where(v => Dates.Count > 0 && v.Start.Date == Dates[0].Date).ToList();

			}

			return await base.LoadDataInternal();
		}
	}
}