﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class CompetitionListViewModel : CancellationViewModel
	{
		private int sportId;

		public CompetitionListViewModel(Sport sport)
		{
			this.Sport = sport;
			this.sportId = sport.Id;
		}

		public CompetitionListViewModel(List<CompetitionItemViewModel> comps)
		{
			this.Competitions = comps;
		}

		public string Name => Sport?.Item.Name;

		public Sport Sport
		{
			get => sport;
			set
			{
				sport = value;OnPropertyChanged();
				this.OnPropertyChanged(nameof(Name));
			}
		}

		public List<CompetitionItemViewModel> Competitions
		{
			get => competitions;
			set { competitions = value; OnPropertyChanged(); }
		}

		public CompetitionItemViewModel SelectedCompetition
		{
			get => null;
			set
			{
				if (value != null)
				{
					OnPropertyChanged();
				}
			}
		}

		private Sport sport;
		private List<CompetitionItemViewModel> competitions;

		protected override async Task<bool> LoadDataInternal()
		{
			var sportManager = new SportManager();
			var compManager = new CompetitionManager();
			List<CompetitionItemViewModel> competitionViewModels = new List<CompetitionItemViewModel>();

			if (Sport == null && this.sportId != 0) // kvůli EF nikdy 0 být nemůže
			{
				using (var managerAssistant = new ManagerAssistant(this, sportManager, "CompetitionListvm_LoadingSport".LocalizeResx()))
				{

					this.Sport = await sportManager.GetById(this.sportId);
				}
			}


			if (Competitions == null)
			{
				List<Competition> comps;
				using (var managerAssistant = new ManagerAssistant(this, compManager, "CompetitionListvm_LoadingCompetitions".LocalizeResx()))
				{
					comps = await compManager.GetBySport(this.Sport);
				}

				foreach (var competition in comps)
				{
					competitionViewModels.Add(new CompetitionItemViewModel(competition, this.Sport));
				}

				this.Competitions = competitionViewModels;
			}


			return await base.LoadDataInternal();
		}
	}
}