﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class CompetitionResultPopupPageViewModel : CancellationViewModel
	{
		private Competition model;

		public string Header => $"{model.Discipline}: {model.Race}";
		public Gender Gender => model.Gender;

		private List<CompetitionResult> results;

		public List<CompetitionResult> Results
		{
			get { return results; }
			set { results = value; OnPropertyChanged(); }
		}

		private List<Region> region;

		public List<Region> Regions
		{
			get { return region; }
			set { region = value; OnPropertyChanged(); }
		}


		private ObservableCollection<ResultItemViewModel> resultVms;

		public ObservableCollection<ResultItemViewModel> ResultViewModels
		{
			get { return resultVms; }
			set { resultVms = value; }
		}

		public ResultItemViewModel SelectedItem
		{
			get { return null; }
			set
			{
				OnPropertyChanged();
			}
		}

		public CompetitionResultPopupPageViewModel(Competition comp)
		{
			this.model = comp;
			this.ResultViewModels = new ObservableCollection<ResultItemViewModel>();
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var competitionResultManager = new CompetitionResultManager();
			var regionManager = new RegionManager();

			using (var managerAssistant = new ManagerAssistant(this, competitionResultManager,"CompetitionResultPopupPageVm_LoadingResults".LocalizeResx()))
			{
				Results = await competitionResultManager.GetByComp(this.model);
				managerAssistant.SetManager(regionManager);
				Regions = await regionManager.GetAll();
			}

			foreach (var competitionResult in Results)
			{
				var region = Regions.Find(r => r.Id == competitionResult.RegionId);
				ResultViewModels.Add(new ResultItemViewModel(competitionResult, region));
			}

			return await base.LoadDataInternal();
		}
	}
}