﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class EventResultPopupPageViewModel :CancellationViewModel
	{
		private Event model;
		private Competition compModel;

		public string Header => $"{compModel?.Discipline}: {compModel?.Race}";

		public string Info => this.model.Title;

		private List<EventResult> results;

		public List<EventResult> Results
		{
			get { return results; }
			set { results = value; OnPropertyChanged(); }
		}

		private List<Region> region;

		public List<Region> Regions
		{
			get { return region; }
			set { region = value; OnPropertyChanged(); }
		}

		private ObservableCollection<ResultItemViewModel> resultVms;

		public ObservableCollection<ResultItemViewModel> ResultViewModels
		{
			get { return resultVms; }
			set { resultVms = value; }
		}

		public ResultItemViewModel SelectedItem
		{
			get { return null; }
			set
			{
				OnPropertyChanged();
			}
		}

		public EventResultPopupPageViewModel(Event eventModel, Competition competition)
		{
			this.model = eventModel;
			this.compModel = competition;
			this.ResultViewModels = new ObservableCollection<ResultItemViewModel>();
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var eventResultManager = new EventResultManager();
			var regionManager = new RegionManager();

			using (var managerAssistant = new ManagerAssistant(this, eventResultManager, "EventResultPopup_LoadingResults".LocalizeResx()))
			{
				Results = await eventResultManager.GetByEvent(this.model);
				managerAssistant.SetManager(regionManager);
				Regions = await regionManager.GetAll();
			}

			foreach (var competitionResult in Results)
			{
				var region = Regions.Find(r => r.Id == competitionResult.RegionId);
				ResultViewModels.Add(new ResultItemViewModel(competitionResult, region));
			}

			return await base.LoadDataInternal();
		}
	}
}