﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.Tools;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Skeleton.Mobile.ODM.Views;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class HomeViewModel : CancellationViewModel
	{
		private List<MessageItemViewModel> messages;

		public List<MessageItemViewModel> Messages
		{
			get { return messages; }
			set { messages = value; OnPropertyChanged(); }
		}

		private List<MessageItemViewModel> AllMessages = new List<MessageItemViewModel>();

		private MessageItemViewModel message;
		private bool _importantIsShowing = false;

		public MessageItemViewModel SelectedItem
		{
			get { return null; }
			set { message = value; }
		}

		private bool isFiltered = false;

		public bool IsFiltered
		{
			get { return isFiltered; }
			set
			{
				isFiltered = value;
				OnPropertyChanged();
			}
		}

		public bool ImportantIsShowing
		{
			get
			{
				return _importantIsShowing;
			}
			set
			{
				_importantIsShowing = value;
				OnPropertyChanged();
			}
		}

		public HomeViewModel()
		{
			this.ShowImportantCommand = new Command(ShowImportantCommand_Execute);
			this.FilterMessagesCommand = new Command(FilterMessagesCommand_Execute);
			this.RefreshCommand = new Command(RefreshCommand_Execute);
		}

		public ICommand ShowImportantCommand { get; set; }

		private void ShowImportantCommand_Execute()
		{
			if (ImportantIsShowing)
			{
				if (filter != MessageTopicEnum.NoTopic)
				{
					this.Messages = AllMessages?.Where(m => m.Topic == filter).ToList();
				}
				else
				{
					this.Messages = AllMessages;
				}
				ImportantIsShowing = false;
			}
			else
			{
				this.Messages = Messages?.Where(m => m.IsImportant == true).ToList();
				ImportantIsShowing = true;
			}
		}

		private MessageTopicEnum filter = MessageTopicEnum.NoTopic;

		private void FilterMessages(MessageTopicEnum topic)
		{
			filter = topic;
			this.Messages = this.Messages?.Where(m => m.Topic == topic).ToList();
		}

		private void UnFilterMessages()
		{
			if (ImportantIsShowing)
			{
				this.Messages = AllMessages?.Where(m => m.IsImportant == true).ToList();
			}
			else
			{
				this.Messages = AllMessages;
			}

			filter = MessageTopicEnum.NoTopic;
		}

		public ICommand FilterMessagesCommand { get; set; }

		private async void FilterMessagesCommand_Execute()
		{
			if (IsFiltered)
			{
				this.DeactivateFilter();
			}
			else
			{
				var filterPage = new MessageListFilterPage();
				filterPage.MessageTopicChanged += (sender, args) =>
				{
					if (args.Topic != MessageTopicEnum.NoTopic)
					{
						IsFiltered = true;
						FilterMessages(args.Topic);
					}
				};
				await RootPage.RootNavigationPage.PushAsync(filterPage, true);
			}
		}

		private void DeactivateFilter()
		{
			this.IsFiltered = false;
			this.UnFilterMessages();
		}

		private bool isRefreshing;

		public bool IsRefreshing
		{	
			get { return isRefreshing; }
			set { isRefreshing = value; OnPropertyChanged(); }
		}
		public ICommand RefreshCommand { get; set; }

		public async void RefreshCommand_Execute()
		{
			this.IsRefreshing = true;

			this.IsFiltered = false;
			this.ImportantIsShowing = false;
			var updateManager = DataUpdateManager.Instance;
			var result = await updateManager.CheckOrUpdateMessages();
			if (result)
			{
				await this.LoadDataInternal();
				OnPropertyChanged(nameof(Messages));
			}

			this.IsRefreshing = false;
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var mesManager = new MessageManager();
			List<Message> messages;
			using (var managerAssistant = new ManagerAssistant(this, mesManager, "HomeViewModel_Loading".LocalizeResx()))
			{
				var mess_all = await mesManager.GetAll();

				messages = mess_all?.OrderByDescending(m => m.Date).Where(m => m.Date < DateTime.Now).ToList();
			}
			if(messages != null){
				foreach (var message in messages)
				{
					var vm = new MessageItemViewModel()
					{
						Date = message.Date,
						IsImportant = message.IsImportant,
						Text = message.Text,
						Title = message.Title,
						Topic = message.Topic,
						IsRemoved = message.IsRemoved,
						Id = message.Id
					};

					if (!AllMessages.Exists(e => e.Id == vm.Id) && !vm.IsRemoved)
					{
						AllMessages.Add(vm);
						AllMessages = AllMessages.OrderByDescending(m => m.Date).ToList();
					}
					else if (vm.IsRemoved)
					{
						AllMessages.Remove(vm);
					}
				}
			}

			Messages = AllMessages;

			return await base.LoadDataInternal();
		}
	}
}
