﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using ODM.Backend.Models.Models.Interfaces;
using ODMWebService.DL.Entities.Enums;
using Rg.Plugins.Popup.Services;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.Abstract;
using Skeleton.Mobile.ODM.Views;
using Skeleton.Mobile.ODM.Views.Abstract;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels.ItemViewModels
{
	public class CompetitionItemViewModel : CompetitionViewModel
	{

		/// <summary>
		/// Pro zobrazení v ProgramByCitiesPage
		/// </summary>
		/// <param name="city">Město, podle kterého se mají zobrazit eventy</param>
		public CompetitionItemViewModel(Competition comp, string city) : this(comp)
		{
			base.city = city;
		}

		/// <summary>
		/// Pro zobrazení ve VenueDetailPage
		/// </summary>
		public CompetitionItemViewModel(Competition comp, Sport sport, Venue venue = null, bool isOnVenue = false) : this(comp)
		{
			base.Sport = sport;
			base.Venue = venue;
			base.IsOnVenuePage = isOnVenue;
		}

		public CompetitionItemViewModel(Competition comp) : base(comp)
		{
			ShowListCommand = new Command(ShowListCommand_Execute);
		}

		#region Commands

		public ICommand ShowListCommand { get; set; }

		public void ShowListCommand_Execute()
		{
			if (Venue == null && city == null)
			{
				ODMPage.SafeGoToPopupPage(new CompetitionDetailPage(new CompetitionDetailViewModel(base.Competition, base.Sport, base.IsOnVenuePage)));
			}
			else if (Venue != null)
			{
				ODMPage.SafeGoToPopupPage(new CompetitionDetailPage(new CompetitionDetailViewModel(base.Competition, base.Sport, base.Venue, base.IsOnVenuePage)));
			}
			else if (city != null)
			{
				ODMPage.SafeGoToPopupPage(new CompetitionDetailPage(new CompetitionDetailViewModel(base.Competition, base.Sport, base.city, base.IsOnVenuePage)));
			}
		}

		#endregion

		protected override async Task<bool> LoadDataInternal()
		{
			var sportManager = new SportManager();

			using (var managerAssistant = new ManagerAssistant(this, sportManager, String.Empty, false))
			{
				if (base.Sport == null)
				{
					base.Sport = await sportManager.GetById(base.SportId);
				}
			}

			return await base.LoadDataInternal();
		}
	}
}