﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Dynamic;
using System.Linq;
using ODM.Backend.Models.Models;
using ODM.Backend.Models.Models.Interfaces;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using System.Windows.Input;
using Xamarin.Forms;
using Skeleton.Mobile.ODM.Views.Abstract;
using System.Threading.Tasks;
using ODMWebService.DL.Entities.Enums;
using Rg.Plugins.Popup.Services;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.Views;

namespace Skeleton.Mobile.ODM.ViewModels.ItemViewModels
{
	public class EventItemViewModel : ViewModel, IEvent
	{
		//TODO: změna interfacu
		public int Id { get; set; }
		public bool IsRemoved { get; set; }

		public DateTime? End { get; set; }
		public string Info { get; set; }
		public DateTime Start { get; set; }
		public int VenueId { get; set; }
		public int CompetitionId { get; set; }
		public string Title { get; set; }

		public Venue Venue { get; private set; }

		public string VenueName => Venue?.Name;

		public string CalendarIcon
		{
			get
			{
				if (IsInCalendar)
				{
					return App.Current.Resources["CalendarMinusIcon"].ToString();
				}
				else
				{
					return App.Current.Resources["CalendarPlusIcon"].ToString();
				}
			}
		}

		public bool IsInCalendar
		{
			get => isInCalendar;
			set { isInCalendar = value; OnPropertyChanged(); OnPropertyChanged(nameof(CalendarIcon)); }
		}

		public string DateSpan
		{
			get
			{
				if (!End.HasValue)
				{
					return $"{Start:t}";
				}
				else return $"{Start:t} - {End.Value:t}";
			}
		}

		public Event eventModel;
		protected Competition compModel;
		private bool isOnVenuePage;
		private bool isInCalendar;

		public bool ShowVenue
		{
			get { return !isOnVenuePage; }
			private set { isOnVenuePage = value; }
		}

		public bool ShowResult { get; private set; }
		public bool ShowInfo { get; private set; }

		public EventItemViewModel(Event @event, Competition comp, bool isOnVenuePage = false, Venue venue = null)
		{
			this.Id = @event.Id;
			this.End = @event.End;
			this.Info = @event.Info;
			this.compModel = comp;
			this.Start = @event.Start;
			this.VenueId = @event.VenueId;
			this.CompetitionId = @event.CompetitionId;
			this.eventModel = @event;
			this.Title = @event.Title;
			this.ShowResultCommand = new Command(ShowResultCommand_Execute);
			this.GoToVenueCommand = new Command(GoToVenueCommand_Execute);
			this.AddToCalendarCommand = new Command(AddToCalendarCommand_Execute);
			this.Venue = venue;
			this.ShowVenue = isOnVenuePage;

			this.ShowResult = compModel.CompetitionType == CompetitionTypeEnum.Main;
			this.ShowInfo = @event.Info != "" || @event.Info != null;
		}

		public ICommand ShowResultCommand { get; set; }

		private void ShowResultCommand_Execute()
		{
			ODMPage.SafeGoToPopupPage(new EventResultPopupPage(this.eventModel, this.compModel));
		}

		public ICommand GoToVenueCommand { get; set; }


		protected async void GoToVenueCommand_Execute()
		{
			if (this.Venue.Location != null)
			{
				string targetTitle = string.Join(", ",
					new List<string>()
					{
						this.Venue.Name,
						this.Venue.Address?.Street,
						this.Venue.Address?.City
					}.Where(x => !string.IsNullOrEmpty(x)));

				var result = await Framework.Device.Device.Instance.Dialogs.ShowDialog(
					"Venue_NavigateTitle".LocalizeResx(), "Venue_NavigateToVenue".LocalizeResx(), "Yes".LocalizeResx(),
					"No".LocalizeResx());
				if (result == Framework.Device.Tools.DialogResult.Accepted)
				{
					await Framework.Device.Device.Instance.OnlineNavigationManager.NavigateFromCurrentLocation(
						new Skeleton.Mobile.Framework.Device.Location.Gps()
						{
							Latitude = this.Venue.Location.Latitude,
							Longitude = this.Venue.Location.Longitude
						}, targetTitle, "Venue_MyLocation".LocalizeResx());
				}
			}
		}

		public ICommand AddToCalendarCommand { get; set; }

		protected async void AddToCalendarCommand_Execute()
		{
			if (IsInCalendar)
			{
				IsInCalendar = false;
				await RemoveFromCalendar();
				ToastNotification.Notify("RemovedFromCalendar".LocalizeResx());
			}
			else
			{
				IsInCalendar = true;
				await AddToCalendar();
				ToastNotification.Notify("AddedToCalendar".LocalizeResx());
			}
		}
		//TODO: Manager assistant asi useless
		protected async Task AddToCalendar()
		{
			var myCalendarManager = MyCalendarManager.Instance;
			myCalendarManager.AddEventToCalendar(this.eventModel);
			await NotificationFactory.CreateNotification(this.eventModel);
		}

		protected async Task RemoveFromCalendar()
		{
			var myCalendarManager = MyCalendarManager.Instance;
			myCalendarManager.RemoveEventFromCalendar(this.eventModel);
			await NotificationFactory.DeleteNotificationByEventId(this.eventModel.Id);
		}
	}
}