﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows.Input;
using Skeleton.Mobile.Framework.Tools.Collections;
using Xamarin.Forms;
using Device = Skeleton.Mobile.Framework.Device.Device;

namespace Skeleton.Mobile.ODM.ViewModels.ItemViewModels
{
	public class GroupItemViewModel<T> : ObservableRangeCollection<T>, INotifyPropertyChanged
	{
		public delegate void GroupExpandChangedEventHandler(object sender, ExpandEventArgs e);
		public event GroupExpandChangedEventHandler GroupExpandChanged;

		public GroupItemViewModel(string name, List<T> group)
		{
			this.Name = FormatName(name);
			this.IconCode = App.Current.Resources["ArrowDownIcon"].ToString();
			this.Expanded = false;
			this.backupList = new ObservableRangeCollection<T>(group);
			this.ExpandCommand = new Command(Expand_Clicked);
		}


		public void Collapse()
		{
			this.IconCode = App.Current.Resources["ArrowDownIcon"].ToString();
			if (backupList?.Count == 0)
				this.backupList.AddRange(this);
			this.Clear();
			Expanded = false;
			GroupExpandChanged?.Invoke(this, new ExpandEventArgs(false, this.Name));
		}

		public void Expand()
		{
			this.IconCode = App.Current.Resources["ArrowUpIcon"].ToString();
			this.AddRange(backupList);
			Expanded = true;
			GroupExpandChanged?.Invoke(this, new ExpandEventArgs(true, this.Name));
		}

		private string FormatName(string name)
		{
			if (DateTime.TryParse(name, out var date))
			{
				var info = Device.Instance.LanguageManager.GetApplicationLanguage();
				var formattedName = name + " " + date.ToString("dddd", info);
				return formattedName;
			}
			else return name;

		}

		#region Properties
		public string Name { get; set; }
		public string IconCode { get; set; }

		public bool Expanded { get; private set; }

		private ObservableRangeCollection<T> backupList;

		public ObservableRangeCollection<T> BackupList
		{
			get { return this.backupList; }
			set
			{
				this.backupList = value;
				this.OnPropertyChanged();
			}
		}

		#endregion

		#region Commands

		public ICommand ExpandCommand { get; private set; }

		private void Expand_Clicked(object obj)
		{
			if (Expanded)
			{
				this.Collapse();
			}
			else
			{
				this.Expand();
			}

			this.OnPropertyChanged(nameof(IconCode));
		}

		#endregion

		//dodaná funkce viewModelu
		public event PropertyChangedEventHandler PropertyChanged;

		protected void OnPropertyChanged([CallerMemberName]string propertyName = "")
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
	}

	//dodatečné infromace k událsoti "změna rozbalení"
	public class ExpandEventArgs : EventArgs
	{
		public ExpandEventArgs(bool expanded, string name)
		{
			this.Expanded = expanded;
			this.Name = name;
		}

		public bool Expanded { get; }

		public string Name { get; }
	}
}