﻿using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Device;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.Abstract;
using Skeleton.Mobile.ODM.Views;
using Skeleton.Mobile.ODM.Views.Abstract;
using Gps = Skeleton.Mobile.Framework.Device.Location.Gps;

namespace Skeleton.Mobile.ODM.ViewModels.ItemViewModels
{
	public class VenueItemViewModel : VenueViewModel
	{
		public VenueItemViewModel(Venue model, Gps gps = null, bool showDistance = true) : base(model)
		{
			currentUserLocation = gps;
			this.showDistance = showDistance;
		}

		private bool showDistance;

		public bool ShowDistance
		{
			get { return showDistance; }
			set
			{
				showDistance = value;
				OnPropertyChanged();
			}
		}

		public string NameWithADash => Name + " - ";


		private Gps currentUserLocation;

		public Gps CurrentUserLocation
		{
			get { return currentUserLocation; }
			set
			{
				currentUserLocation = value;
				OnPropertyChanged();
				OnPropertyChanged(nameof(Distance));
				OnPropertyChanged(nameof(FormattedDistance));
			}
		}

		private float? Distance
		{
			get
			{
				if (CurrentUserLocation != null && VenueLocation != null)
				{
					return Device.Instance.LocationManager.DistanceBetweenPoints(new Gps()
					{
						Latitude = VenueLocation.Latitude,
						Longitude = VenueLocation.Longitude
					}, CurrentUserLocation);
				}
				else return null;
			}
		}

		public string FormattedDistance
		{
			get
			{
				if (Distance.HasValue)
				{
					if (Distance > 1000)
					{
						var distanceInKm = ((double)(Distance / 1000)).ToString("0.0") + " km";

						return distanceInKm;
					}
					else return Distance + " m";
				}

				if (CurrentUserLocation != null)
				{
					return null;
				}
				else
				{
					return "VenueItem_Location".LocalizeResx();
				}
			}
		}

		public void OpenDetail()
		{
			ODMPage.SafeGoToPage(new VenueDetailPage(base.Model));
		}
	}
}