﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.Views;
using Skeleton.Mobile.ODM.Views.Abstract;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels
{
	class MapViewModel : CancellationViewModel
	{
		public event EventHandler VenuesReloaded;

		private bool loadSelf;
		private bool filter;

		public MapViewModel(List<Venue> venues, bool filter)
		{
			loadSelf = venues == null;
			this.filter = filter;
			if (!loadSelf)
			{
				Venues = venues;
				this.OnVenuesReloaded();
			}

			this.ShowVenueListCommand = new Command(this.ShowVenueListClicked);
		}

		protected override async Task<bool> LoadDataInternal()
		{
			await base.LoadDataInternal();

			if (!loadSelf)
			{
				return true;
			}

			List<Venue> venues = null;
			bool result;
			var venuesManager = new VenueManager();

			this.SetBackgroundWorking(true, "Map_Loading".LocalizeResx());
			using (var assistant = new ManagerAssistant(this, venuesManager, String.Empty, false))
			{
				venues = (List<Venue>)await venuesManager.GetAll();
				result = venues != null;
			}
			if (result)
			{
				this.Venues = venues;
				
				this.OnVenuesReloaded();
			}

			this.SetBackgroundWorking(false);

			return result;
		}

		public async Task RefreshVenues()
		{
			if (this.Venues != null)
			{
				this.OnVenuesReloaded();
			}
		}

		private void OnVenuesReloaded()
		{
			this.VenuesReloaded?.Invoke(this, EventArgs.Empty);
		}

		#region Properties

		private List<Venue> venuesData;

		public List<Venue> Venues
		{
			get { return this.venuesData; }
			set
			{
				this.venuesData = value;
				this.OnPropertyChanged();
			}
		}

		#endregion

		#region Commands

		public ICommand ShowVenueListCommand { get; private set; }
		

		private async void ShowVenueListClicked()
		{
			ODMPage.SafeGoToPage(new VenueListPage(this.Venues, filter));
		}

		#endregion
	}
}
