﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.Framework.Error.Log;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Skeleton.Mobile.ODM.Views;
using Xamarin.Forms.Internals;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class MenuViewModel : BackgroundViewModel
	{
		public MenuViewModel()
		{
			this.MenuItems = new ObservableCollection<MenuItemViewModel>();
			RootPage.GetMenuItems.ForEach(item => this.MenuItems.Add(item));
		}

		private void MarkItemAsSelected(MenuItemViewModel newSelectedItem)
		{
			this.selectedItem = newSelectedItem;
			foreach (var item in MenuItems)
			{
				if (item == selectedItem)
				{
					item.Selected = true;
				}
				else
				{
					item.Selected = false;
				}
			}
		}

		#region Public methods

		public void MarkItemAsSelected(Type itemPageType)
		{
			var newSelectedItem = this.MenuItems.SingleOrDefault(x => x.TargetType == itemPageType);
			if (newSelectedItem != null)
				this.MarkItemAsSelected(newSelectedItem);
			else
				ErrorLog.Instance.StartAddRecord(ErrorTypes.WARNING, $"Type - {itemPageType}", "MenuViewModel.MarkItemAsSelected", "NewSelectedItem is null");
		}

		#endregion

		#region Properties
		private ObservableCollection<MenuItemViewModel> menuItems;
		public ObservableCollection<MenuItemViewModel> MenuItems
		{
			get
			{
				return menuItems;
			}
			private set
			{
				menuItems = value;
				this.OnPropertyChanged(nameof(MenuItems));
			}
		}

		private MenuItemViewModel selectedItem;
		public MenuItemViewModel SelectedItem
		{
			get
			{
				return this.selectedItem;
			}
			set
			{
				this.selectedItem = value;
				this.MarkItemAsSelected(this.selectedItem);
				OnPropertyChanged();
			}
		}

		#endregion
	}
}