﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class MyCalendarRootViewModel : CancellationViewModel
	{
		private List<Event> Events { get; set; }
		private List<Competition> Competitions { get; set; }
		private List<Sport> Sports{ get; set; }
		private List<Venue> Venues { get; set; }

		public List<EventItemCalendarViewModel> CalendarEvents = new List<EventItemCalendarViewModel>();

		public List<DateTime> Dates = new List<DateTime>();

		protected override async Task<bool> LoadDataInternal()
		{
			var myCalendarManager = MyCalendarManager.Instance;
			var competitionManager = new CompetitionManager();
			var sportManager = new SportManager();
			var venueManager = new VenueManager();

			List<int> competitionIds = new List<int>();

			using (var managerAssistant = new ManagerAssistant(this, myCalendarManager, "MyCalendarRoot_LoadingCalendar".LocalizeResx(), false))
			{
				this.Events = await myCalendarManager.GetCalendarEvents();
				foreach (var @event in Events)
				{
					 int compId = @event.CompetitionId;

					if (!Dates.Contains(@event.Start.Date))
					{
						Dates.Add(@event.Start.Date);
					}

					if (!competitionIds.Contains(compId))
					{
						competitionIds.Add(compId);
					}
				} // Eventy a Datumy
				Dates.Sort();
				managerAssistant.SetManager(competitionManager);

				var comps = await competitionManager.GetAll();

				List<Competition> calendarComps = new List<Competition>();
				foreach (var competitionId in competitionIds)
				{
					Competition comp = comps.Find(c => c.Id == competitionId);
					calendarComps.Add(comp);
				}

				Competitions = calendarComps;

				managerAssistant.SetManager(sportManager);
				this.Sports = await sportManager.GetAll();

				managerAssistant.SetManager(venueManager);
				this.Venues = await venueManager.GetAll();
			}

			foreach (var @event in this.Events)
			{
				var comp = Competitions.Find(c => c.Id == @event.CompetitionId);
				var venue = Venues.Find(v => v.Id == @event.VenueId);
				var sport = Sports.Find(s => s.Id == comp.SportId);
				EventItemCalendarViewModel calVm = new EventItemCalendarViewModel(@event, comp, venue, sport);
				this.CalendarEvents.Add(calVm);
			}

			return await base.LoadDataInternal();
		}
	}
}