﻿using System.Windows.Input;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.Framework.Device.Tools;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class NavigateHomeViewModel : CancellationViewModel
	{
		private bool locationSet;
		private string address;
		public ICommand AboutPageCommand { get; set; }
		public ICommand NavigateCommand { get; set; }
		public ICommand LocationResetCommand { get; set; }

		public delegate void IconClicked();

		public IconClicked Clicked { get; set; }

		public bool LocationSet
		{
			get { return locationSet; }
			set { locationSet = value; OnPropertyChanged(); }
		}

		public string Icon
		{
			get
			{
				if (LocationSet)
				{
					return "\uf015";
				}
				else return "\uf3c5";
			}
		}

		public string Address
		{
			get => address;
			set { address = value; OnPropertyChanged(); }
		}

		public NavigateHomeViewModel()
		{
			this.AboutPageCommand = new Command(AboutPageCommand_Execute);
			this.NavigateCommand = new Command(Navigate_Execute);
			this.LocationResetCommand = new Command(LocationReset_Execute);

			var locManager = LocationManager.Instance;

			locationSet = locManager.GetLocationWasSet;

			if (!locationSet)
			{
				this.Address = "NavigateHomePage_NoLocation".LocalizeResx()+"\n\n";
			}
			else
			{
				this.Address = locManager.GetAddress();
			}

		}



		private async void LocationReset_Execute()
		{
			var result = await Framework.Device.Device.Instance.Dialogs.ShowDialog(
				"", "NavigateHomePage_WannaReset".LocalizeResx(), "Yes".LocalizeResx(),
				"No".LocalizeResx());

			if (result != DialogResult.Accepted)
			{
				return;
			}

			var locManager = LocationManager.Instance;
			Gps gps = await locManager.SetHousingGps();
			var wasFound = await locManager.SetHousingLocation(gps);
			if (wasFound)
			{
				this.LocationSet = locManager.GetLocationWasSet;
				Address = locManager.GetAddress();
				Framework.Device.Device.Instance.Dialogs.ShowToast("NavigateHomePage_LocationSet".LocalizeResx(), Length.Short);
				Clicked?.Invoke();
			}
			else
			{
				Skeleton.Mobile.Framework.Device.Device.Instance.Dialogs.ShowDialog("LocationNotFound_Title".LocalizeResx(), "LocationNotFound_Text".LocalizeResx());
			}
		}

		private async void Navigate_Execute()
		{
			var allowed = await Skeleton.Mobile.Framework.Device.Device.Instance.PermissionsManager
				.RequestLocationInUsePermission();

			if (!allowed)
			{
				Skeleton.Mobile.Framework.Device.Device.Instance.Dialogs.ShowDialog("NotAllowed".LocalizeResx(),
					"NavigateHomePage_NotAllowed".LocalizeResx());
				return;
			}

			if (!LocationSet)
			{
				var locManager = LocationManager.Instance;
				bool set;

				Gps gps = await locManager.SetHousingGps();

				if(gps!=null){
					set = await locManager.SetHousingLocation(gps);
					if (set)
					{
						this.LocationSet = locManager.GetLocationWasSet;
						Framework.Device.Device.Instance.Dialogs.ShowToast(
							"NavigateHomePage_LocationSet".LocalizeResx(), Length.Short);
						Clicked?.Invoke();
						this.Address = locManager.GetAddress();
					}
					else
					{
						Skeleton.Mobile.Framework.Device.Device.Instance.Dialogs.ShowDialog("LocationNotFound_Title".LocalizeResx(), "LocationNotFound_Text".LocalizeResx());
					}
				}

				
			}
			else
			{
				var locManager = LocationManager.Instance;
				Gps location;

				location = locManager.GetHousingLocation();


				var result = await Framework.Device.Device.Instance.Dialogs.ShowDialog(
					"Venue_NavigateTitle".LocalizeResx(), "Venue_NavigateToVenue".LocalizeResx(), "Yes".LocalizeResx(),
					"No".LocalizeResx());
				if (result == Framework.Device.Tools.DialogResult.Accepted)
				{
					await Framework.Device.Device.Instance.OnlineNavigationManager.NavigateFromCurrentLocation(
						location, "NavigateHomePage_Home".LocalizeResx());
				}
			}
		}

		private void AboutPageCommand_Execute()
		{
			Skeleton.Mobile.Framework.Device.Device.Instance.Dialogs.ShowDialog("",
				"NavigateHomePage_About".LocalizeResx());
		}
	}
}