﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;

namespace Skeleton.Mobile.ODM.ViewModels
{
	class ProgramByCityViewModel : CancellationViewModel
	{
		public List<String> Cities { get; private set; }
		public Dictionary<string, List<CompetitionItemViewModel>> CityCompetitionVmDictionary { get; private set; }

		public ProgramByCityViewModel()
		{
			CityCompetitionVmDictionary = new Dictionary<string, List<CompetitionItemViewModel>>();
			Cities = new List<string>();
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var venueManager = new VenueManager();
			var competitionManager = new CompetitionManager();

			using (var managerAssistant = new ManagerAssistant(this, venueManager, "ProgramByCityvm_LoadingCompetitions".LocalizeResx()))
			{
				Cities = await venueManager.GetCities() ?? new List<string>();

				managerAssistant.SetManager(competitionManager);
				foreach (var city in Cities)
				{
					var comps = await competitionManager.GetByCity(city) ?? new List<Competition>();
					CityCompetitionVmDictionary[city] = new List<CompetitionItemViewModel>();
					foreach (var comp in comps)
					{
						CityCompetitionVmDictionary[city].Add(new CompetitionItemViewModel(comp, city));
					}
				}
			}


			return await base.LoadDataInternal();
		}
	}
}
