﻿using System.Collections.Generic;
using System.Drawing;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.Abstract;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Skeleton.Mobile.ODM.Views;
using Skeleton.Mobile.ODM.Views.Abstract;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class ProgramBySportsViewModel : TilesViewModel
	{
		public ProgramBySportsViewModel() : base()
		{
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var sportsManager = new SportManager();

			List<Sport> sports;
			using (var managerAssistant = new ManagerAssistant(this, sportsManager, "ProgramBySports_LoadingSports".LocalizeResx()))
			{
				sports = await sportsManager.GetAll();
			}

			var sportTiles = CreateSportTiles(sports);
			base.TileItems = sportTiles;
			return await base.LoadDataInternal();
		}

		private List<TileItemViewModel> CreateSportTiles(List<Sport> sports)
		{
			var output = new List<TileItemViewModel>();
			if(sports != null)
			{
				foreach (var sport in sports)
				{
					output.Add(new TileItemViewModel(
						title: sport.Item.Name,
						key: sport.Item.Key,
						color: Color.Blue,
						command: () => { ODMPage.SafeGoToPage(new CompetitionListPage(sport)); })
					);
				}
			}

			return output;
		}
	}
}