﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.Abstract;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Gps = Skeleton.Mobile.Framework.Device.Location.Gps;

namespace Skeleton.Mobile.ODM.ViewModels
{
	class VenueDetailViewModel : VenueViewModel
	{
		public VenueDetailViewModel(Venue model) : base(model)
		{
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var competitionManager = new CompetitionManager();
			var sportsManager = new SportManager();
			List<Competition> comps;
			List<Sport> sports;
			using (var managerAssistant = new ManagerAssistant(this, competitionManager, "VenueDetail_LoadingCompetitions".LocalizeResx()))
			{
				comps= await competitionManager.GetByVenue(this.Model);
				managerAssistant.SetManager(sportsManager);
				sports = await sportsManager.GetAll();
			}

			List<CompetitionItemViewModel> compViewModels = new List<CompetitionItemViewModel>();
			foreach (var competition in comps)
			{
				var sport = sports.Find(s => s.Id == competition.SportId);
				compViewModels.Add(new CompetitionItemViewModel(competition, sport, this.Model, true));
			}

			this.Competitions = compViewModels;

			return await base.LoadDataInternal();
		}

		#region Properties

		private List<CompetitionItemViewModel> competitions;

		public List<CompetitionItemViewModel> Competitions
		{
			get { return this.competitions; }
			private set
			{
				this.competitions = value;
				this.OnPropertyChanged();
			}
		}

		public CompetitionItemViewModel SelectedCompetition
		{
			get { return null; }
			set
			{
				if (value != null)
				{
					OnPropertyChanged();
				}
			}
		}

		public bool ShowNavigation => this.Model.Location != null;

		#endregion

		#region Commands

		public async void NavigateClick()
		{
			if (this.Model.Location != null)
			{
				string targetTitle = string.Join(", ",
					new List<string>()
					{
						this.Model.Name,
						this.Model.Address?.Street,
						this.Model.Address?.City
					}.Where(x => !string.IsNullOrEmpty(x)));

				var result = await Framework.Device.Device.Instance.Dialogs.ShowDialog(
					"Venue_NavigateTitle".LocalizeResx(), "Venue_NavigateToVenue".LocalizeResx(), "Yes".LocalizeResx(),
					"No".LocalizeResx());
				if (result == Framework.Device.Tools.DialogResult.Accepted)
				{
					await Framework.Device.Device.Instance.OnlineNavigationManager.NavigateFromCurrentLocation(
						new Gps()
						{
							Latitude = this.Model.Location.Latitude,
							Longitude = this.Model.Location.Longitude
						}, targetTitle, "Venue_MyLocation".LocalizeResx());
				}
			}
		}
		#endregion

	}
}