﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class VenueSearchViewModel : CancellationViewModel
	{
		public VenueSearchViewModel()
		{
			this.Venues = new ObservableCollection<VenueItemViewModel>();
		}

		private string query;

		public string Query
		{
			get => query;
			set { this.query = value; OnPropertyChanged();
				if (Query.Length >= 2)
				{
					QueryTask();
				}
				else
				{
					Venues.Clear();
				}}
		}

		private ObservableCollection<VenueItemViewModel> venues;

		public ObservableCollection<VenueItemViewModel> Venues
		{
			get { return venues; }
			set { venues = value; OnPropertyChanged(); }
		}

		public VenueItemViewModel SelectedItem
		{
			get { return null; }
			set
			{
				if (value != null)
				{
					value.OpenDetail();
					OnPropertyChanged();
				}
			}
		}

		public List<VenueItemViewModel> VenuesBackingList { get; private set; }

		private async Task QueryTask()
		{
			foreach (var venue in VenuesBackingList)
			{
				var name = venue.Name;
				var address = venue.Address;
				var querySuccessful = false;

				await Task.Run(() =>
				{
					querySuccessful |= StringSimilarityComparison.AreSimilar(Query, name);
					querySuccessful |= StringSimilarityComparison.AreSimilar(Query, address);
				});

				if (querySuccessful && !Venues.Contains(venue))
				{
					Venues.Add(venue);
				}
				else if(!querySuccessful)
				{
					if (Venues.Contains(venue))
					{
						Venues.Remove(venue);
					}
				}
			}
		}

		protected override async Task<bool> LoadDataInternal()
		{
			var venueManager = new VenueManager();

			List<Venue> venueModels;
			using (var managerAssistant = new ManagerAssistant(this, venueManager, "", false))
			{
				venueModels = await venueManager.GetAll();
			}
			List<VenueItemViewModel> venueViewModels = new List<VenueItemViewModel>();
			foreach (var venueModel in venueModels)
			{
				venueViewModels.Add(new VenueItemViewModel(venueModel, null, false));
			}

			VenuesBackingList = venueViewModels;

			return await base.LoadDataInternal();
		}
	}
}