﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.ODM.Backend.Managers;
using Skeleton.Mobile.ODM.Converters;
using Skeleton.Mobile.ODM.Models;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ComponentViewModels;
using Skeleton.Mobile.ODM.Views;
using Skeleton.Mobile.ODM.Views.Abstract;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class VenuesFilterViewModel : CancellationViewModel
	{
		private List<string> Cities { get; set; } = null;
		private List<Sport> Sports { get; set; } = null;

		private List<VenueTypeEnum> VenueTypes;

		public List<MultiSelectItem> SelectedCities = new List<MultiSelectItem>();
		public List<MultiSelectItem> SelectedSports = new List<MultiSelectItem>();
		public List<MultiSelectItem> SelectedTypes = new List<MultiSelectItem>();

		public VenuesFilterViewModel()
		{
			this.VenueTypes = new List<VenueTypeEnum>()
			{
				VenueTypeEnum.Sport,
				VenueTypeEnum.Housing,
				VenueTypeEnum.Canteen
			};
		}

		protected override async Task<bool> LoadDataInternal()
		{
			await base.LoadDataInternal();

			VenueManager vManager = new VenueManager();
			SportManager sManager = new SportManager();
			using (var managerAssistant = new ManagerAssistant(this, vManager, String.Empty, false))
			{
				Cities = await vManager.GetCities();

				managerAssistant.SetManager(sManager);
				this.Sports = await sManager.GetAll();
			}

			this.SetBackgroundWorking(false);

			return Cities != null;
		}
			
		/// <summary>
		/// Podle zadaných filtrů vybere seznam míst a zobrazí na mapě
		/// </summary>
		/// TODO: filtrovací metody zapouzdřit vedle, jestli se mi bude chtít
		public async Task ShowOnMap_Execute()
		{
			List<Venue> venues;
			List<Sport> sports;
			List<VenueTypeEnum> types = new List<VenueTypeEnum>();

			var venueManager = new VenueManager();
			var sportManager = new SportManager();

			using (var managerAssistant = new ManagerAssistant(this, venueManager, "", false))
			{
				//pokud nejsou vybrané sporty, list se zaplní všemi místy
				if (SelectedSports.Count == 0)
				{
					venues = await venueManager.GetAll();
				}
				//pokud jsou vybrané sporty, list se zaplní místy, kde se tyto sporty hrají
				else
				{
					managerAssistant.SetManager(sportManager);
					sports = await sportManager.GetAll();

					var filtSports = new List<Sport>();
					foreach (var sport in SelectedSports)
					{
						filtSports.Add(sports.Find(s => s.Item.Name == sport.DisplayName));
					}
					sports = filtSports;

					managerAssistant.SetManager(venueManager);
					venues = new List<Venue>();

					foreach (var sport in sports)
					{
						venues.AddRange(await venueManager.GetBySport(sport));
					}
					// díky několika výběrům z repository budou v listu duplikáty, odstranění přes linq
					venues = venues.Distinct().ToList();
				}
			}

			if (SelectedTypes.Count != 0)
			{
				var conv = new VenueTypeToNameConverter();

				foreach (var type in SelectedTypes)
				{
					//jelikož je ten multiselect debilní a podporuje jen Displayname, musí se názvy enumů převést zpátky na enum
					types.Add((VenueTypeEnum)conv.ConvertBack(type.DisplayName, null,null,null));
				}
			}

			List<Venue> fVenues = new List<Venue>();
			if (venues != null)
			{
				foreach (var venue in venues)
				{
					bool city = SelectedCities.Count == 0; //pokud žádné nejsou, automaticky venue projde tímto filtrem
					bool typ = types.Count == 0; //pokud žádné nejsou, automaticky venue projde tímto filtrem

					foreach (var selectedCity in SelectedCities)
					{
						if (!city) //pokud je venue ve městě, které je filtrované, prochází filtrem
						{
							city = venue.Address.City == selectedCity.DisplayName;
						}
					}

					foreach (var selectedType in types)
					{
						if (!typ) //pokud je typ venue ten, který je filtrovaný, prochází filtrem
						{
							typ = venue.Type == selectedType;
						}
					}

					if ((city & typ))
					{
						fVenues.Add(venue);
					}
				}
			}

			if (fVenues.Count == 0)
			{
				ToastNotification.Notify("PlacesSearchPage_NoneFound".LocalizeResx());
			}
			else
			{
				ODMPage.SafeGoToPage(new VenueListPage(fVenues, false));
			}
		}

		public MultiSelectViewViewModel CreateSportVm()
		{
			List<string> items = new List<string>();
			if(Sports != null)
			{
				foreach (var sport in Sports)
				{
					items.Add(sport.Item.Name);
				}
			}
			var filterBySportsVm = new MultiSelectViewViewModel(items, new List<bool>());
			return filterBySportsVm;
		}

		public MultiSelectViewViewModel CreateCityVm()
		{
			List<string> items = new List<string>();
			if(Cities != null){
				foreach (var city in this.Cities)
				{
					items.Add(city);
				}
			}
			
			var filterByCityvM = new MultiSelectViewViewModel(items, new List<bool>());

			return filterByCityvM;
		}

		public MultiSelectViewViewModel CreateTypeVm()
		{
			List<string> items = new List<string>();

			foreach (var type in this.VenueTypes)
			{
				var cnv = new VenueTypeToNameConverter();
				var name = (string)cnv.Convert(type, null, null, null);

				items.Add(name);
			}

			var filterByTypeVm = new MultiSelectViewViewModel(items, new List<bool>());
			return filterByTypeVm;
		}
	}
}