﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Skeleton.Mobile.ODM.ViewModels.ComponentViewModels;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Skeleton.Mobile.ODM.Views.CustomViews;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace Skeleton.Mobile.ODM.Views.Components
{
	[XamlCompilation(XamlCompilationOptions.Compile)]
	public partial class EventCalendarView : ContentView
	{
		private EventCalendarViewModel vm;
		public EventCalendarView (EventCalendarViewModel vm)
		{
			InitializeComponent ();
			this.vm = vm;
			this.HorizontalOptions = LayoutOptions.FillAndExpand;
			this.VerticalOptions = LayoutOptions.FillAndExpand;
		}

		private Grid grid;

		public void CreateCalendar()
		{
			if (this.vm.HourlyEvents.Count == 0)
			{
				var label = new Label()
				{
					Text = "Nic tu není", //TODO
					HorizontalTextAlignment = TextAlignment.Center,
					VerticalTextAlignment = TextAlignment.Center,
					VerticalOptions = LayoutOptions.Center,
					HorizontalOptions = LayoutOptions.Center
				};
				this.CalendarStack.Children.Add(label);
				return;
			}

			grid = new Grid();
			grid.ColumnDefinitions.Add(new ColumnDefinition() { Width = GridLength.Auto });
			grid.ColumnDefinitions.Add(new ColumnDefinition() { Width = GridLength.Auto });

			this.CalendarStack.Children.Add(grid);

			int i = 0;
			foreach (KeyValuePair<string, List<EventItemCalendarViewModel>> hourEventsGroup in vm.HourlyEvents)
			{
				i++;
				this.grid.RowDefinitions.Add(new RowDefinition() { Height = GridLength.Auto });
				DrawEvents(hourEventsGroup.Value, i);
				CreateTimeHeader(hourEventsGroup.Key, i);
			}
		}

		protected void CreateTimeHeader(string time, int gridRow)
		{
			var timeLabel = new Label() { Text = time, HorizontalTextAlignment = TextAlignment.Center, Margin = new Thickness() { Left = 5d } };
			BoxView separator = new BoxView
			{
				Color = Color.Black,
				HorizontalOptions = LayoutOptions.FillAndExpand,
				WidthRequest = 1,
				Margin = new Thickness() { Top = 20d, Left = 7d }
			};
			timeLabel.SetValue(Grid.RowProperty, gridRow);
			timeLabel.SetValue(Grid.ColumnProperty, 0);

			separator.SetValue(Grid.RowProperty, gridRow);
			separator.SetValue(Grid.ColumnProperty, 0);
			separator.HorizontalOptions = LayoutOptions.Center;


			this.grid.Children.Add(timeLabel);
			this.grid.Children.Add(separator);
		}

		protected void DrawEvents(List<EventItemCalendarViewModel> events, int gridRow)
		{
			var stack = new StackLayout();
			stack.SetValue(Grid.RowProperty, gridRow);
			stack.SetValue(Grid.ColumnProperty, 1);

			foreach (var eventVm in events)
			{
				Frame eventFrame = new Frame();
				var view = new EventItemCalendarView(eventVm);
				eventFrame.Content = view;
				eventFrame.BackgroundColor = (Color)App.Current.Resources["SecondaryBackgroundColor"];
				eventFrame.HasShadow = false;
				if (events.IndexOf(eventVm) == 0)
				{
					eventFrame.Margin = new Thickness() { Top = 20d, Bottom = 0, Left = 0, Right = (Double)App.Current.Resources["DefaultMarginValue"] };
				}
				else
				{
					eventFrame.Margin = new Thickness() { Top = 0, Bottom = 0, Left = 0, Right = (Double)App.Current.Resources["DefaultMarginValue"] };
				}
				eventFrame.Padding = 0;
				eventFrame.CornerRadius = 2;

				stack.Children.Add(eventFrame);
				this.grid.Children.Add(stack);
			}
		}
	}
}