﻿using System;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.Views.CustomViews
{
	/// <summary>
	/// Vlastní komponenta tlačítka, které rozšiřuje ContentView o klikací animaci
	/// </summary>
	public class ContentButton : ContentView
	{
		public static readonly BindableProperty CommandProperty = BindableProperty.Create(nameof(Command), typeof(ICommand), typeof(ContentButton), null);
		public ICommand Command
		{
			get { return (ICommand)GetValue(CommandProperty); }
			set { SetValue(CommandProperty, value); }
		}

		public static readonly BindableProperty CommandParameterProperty = BindableProperty.Create(nameof(CommandParameter), typeof(object), typeof(ContentButton), null);
		public object CommandParameter
		{
			get { return (object)GetValue(CommandParameterProperty); }
			set { SetValue(CommandParameterProperty, value); }
		}

		/// <summary>
		/// Událost která se volá, pokud dojde ke kliknutí na toto tlačítko
		/// </summary>
		public event EventHandler Clicked;

		private void OnClicked()
		{
			this.Clicked?.Invoke(this, EventArgs.Empty);
		}

		public ContentButton()
		{
			Initialize();
			AnimationDuration = 100;
		}

		public void Initialize()
		{
			GestureRecognizers.Add(new TapGestureRecognizer()
			{
				Command = TransitionCommand
			});
		}

		private ICommand TransitionCommand
		{
			get
			{
				return new Command(async () =>
				{
					RootPage.SetIgnoreBackButton(true); //v průbehu animace ignorujeme tlačítko zpět (vypínalo by apliakci a působilo dojmem pádu apolikace)
					this.AnchorX = 0.48;
					this.AnchorY = 0.48;
					await this.ScaleTo(0.9, 50, Easing.Linear);
					await Task.Delay(AnimationDuration);
					await this.ScaleTo(1, 50, Easing.Linear);
					this.OnTouch();
					RootPage.SetIgnoreBackButton(false);
				});
			}
		}

		private void OnTouch()
		{
			if (Command != null)
			{
				Command.Execute(CommandParameter);
			}
			this.OnClicked();
		}

		/// <summary>
		/// Určuje rychlost animace
		/// </summary>
		public int AnimationDuration { get; set; }
	}
}
