﻿using System;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.Framework.Data.Configuration;
using Skeleton.Mobile.ODM.Backend.Managers;
using Xamarin.Forms.Xaml;
using Skeleton.Mobile.ODM.Views.Abstract;

namespace Skeleton.Mobile.ODM.Views
{
	[XamlCompilation(XamlCompilationOptions.Compile)]
	public partial class MaintenancePage : ODMPage
	{
		private MaintenancePageViewModel viewModel;
		private string staticDataVersion;
		private string messageDataVersion;
		private string resultDataVersion;
		private string programDataVersion;

		public string StaticDataVersion
		{
			get => $"Verze statických dat: {staticDataVersion}";
			set => staticDataVersion = value;
		}

		public string MessageDataVersion
		{
			get => $"Verze zpráv: {messageDataVersion}";
			set => messageDataVersion = value;
		}

		public string ProgramDataVersion
		{
			get => $"Verze dat programu: {programDataVersion}";
			set => programDataVersion = value;
		}

		public string ResultDataVersion
		{
			get => $"Verze dat výsledků: {resultDataVersion}";
			set => resultDataVersion = value;
		}

		public MaintenancePage(AbsConfigurationFactory configuration)
		{
			InitializeComponent();
			this.viewModel = new MaintenancePageViewModel(configuration);
			this.BindingContext = this.viewModel;
		}

		protected override async void OnAppearing()
		{
			base.OnAppearing();
			await viewModel.LoadData();
			var settingsManager = SettingsManager.Instance;
			this.StaticDataVersion = settingsManager.StaticVersion.ToString();
			this.MessageDataVersion = settingsManager.MessageVersion.ToString();
			this.ProgramDataVersion = settingsManager.ProgramVersion.ToString();
			this.ResultDataVersion = settingsManager.ResultsVersion.ToString();

			this.Static.Text = StaticDataVersion;
			this.Message.Text = MessageDataVersion;
			this.Program.Text = ProgramDataVersion;
			this.Result.Text = ResultDataVersion;
		}

		private async void StaticBtn_OnClicked(object sender, EventArgs e)
		{
			StaticDataVersion = (await DataManager.ClearStaticData(true)).ToString();
			this.Static.Text = StaticDataVersion;
		}

		private async void MessageBtn_OnClicked(object sender, EventArgs e)
		{
			MessageDataVersion = (await DataManager.ClearMessageData(true)).ToString();
			this.Message.Text = MessageDataVersion;
		}

		private async void ProgramBtn_OnClicked(object sender, EventArgs e)
		{
			ProgramDataVersion = (await DataManager.ClearProgramData(true)).ToString();
			this.Program.Text = ProgramDataVersion;
		}

		private async void ResultBtn_OnClicked(object sender, EventArgs e)
		{
			ResultDataVersion = (await DataManager.ClearResultData(true)).ToString();
			this.Result.Text = ResultDataVersion;
		}
	}
}