﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.Map.Map;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.ODM.Backend.Enums;
using Skeleton.Mobile.ODM.Converters;
using Skeleton.Mobile.ODM.Extensions;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels;
using Skeleton.Mobile.ODM.Views.Abstract;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using Gps = Skeleton.Mobile.Framework.Device.Location.Gps;

namespace Skeleton.Mobile.ODM.Views
{
	[XamlCompilation(XamlCompilationOptions.Compile)]
	public partial class MapPage : ODMPage
	{
		private MapViewModel viewModel;
		private bool isFirstAppearing;
		private bool initialZoomDone;

		public MapPage(List<Venue> venues = null, bool filter = true)
		{
			InitializeComponent();
			this.viewModel = new MapViewModel(venues, filter);
			this.viewModel.VenuesReloaded += ViewModel_VenuesReloaded;
			if (venues != null)
			{
				this.SetData();
			}
			this.BindingContext = viewModel;
			this.isFirstAppearing = true;
			this.initialZoomDone = false;

				//< ToolbarItem Text = "{extensions:Translate Map_ShowVenueList}" Icon = "list_icon.png" Command = "{Binding ShowVenueListCommand}" />
			if (filter)
			{
				var showMapToolbar = new ToolbarItem();
				showMapToolbar.Text = "Map_ShowVenueList".LocalizeResx();
				showMapToolbar.Icon = "list_icon.png";
				showMapToolbar.Command = viewModel.ShowVenueListCommand;
				this.ToolbarItems.Add(showMapToolbar);
			}
		}

		protected override async void OnAppearing()
		{
			base.OnAppearing();
			this.Map.MapCustomItemsLoaded += Map_MapCustomItemsLoaded;

			if (await Framework.Device.Device.Instance.PermissionsManager.GetLocationInUsePermission())
			{
				this.Map.IsShowingUser = true;
			}

			// pokud se stránka nezobrazuje poprvé, tak se provede aktualizace venues pro případ, kdy by došlo k přidání screeningu - změna barvy pinu
			if (this.isFirstAppearing)
			{
				this.isFirstAppearing = false;
			}
			else
			{
				await this.viewModel.RefreshVenues();
			}
		}

		private void Map_MapCustomItemsLoaded(object sender, EventArgs e)
		{
			// aby se nepřezoomovávalo na piny po jejich aktualizaci při vrácení se ze stránky seznamů promítání
			if (!initialZoomDone)
			{
				Map?.ZoomOnAll();
				initialZoomDone = true;
			}
		}

		protected override void OnDisappearing()
		{
			// mapa si sama neukládá poslední zoom provedený uživatelem, proto se při odchodu stránky volá MoveToRegion
			// Forms mapa si to pak uloží a automaticky visible region načte při dalším zobrazení
			if (this.Map.VisibleRegion != null) //bývá null při prvním zobrazení mapy po instalaci aplikace a opuštení stránky bez předchozího posunutí mapy
			{
				this.Map.MoveToRegion(this.Map.VisibleRegion);
			}
			this.Map.IsShowingUser = false;
			base.OnDisappearing();
			this.Map.MapCustomItemsLoaded -= Map_MapCustomItemsLoaded;
		}

		private void ViewModel_VenuesReloaded(object sender, EventArgs args)
		{
			this.SetData();
		}

		private Color? SetPinColor(VenueTypeEnum type)
		{
			VenueTypeToColorConverter conv = new VenueTypeToColorConverter();
			return (Color?)conv.Convert(type, null, null, null);
		}

		private void SetData()
		{
			if (viewModel.Venues != null)
			{
				if (this.Map != null)
				{
					this.Map.Pins.Clear();
					this.Map.CustomPins.Clear();

					foreach (var venue in viewModel.Venues)
					{
						if (venue.Location != null)
						{
							MapPin p = new MapPin();
							p.Title = venue.Name;
							p.Position = new Gps()
							{
								Latitude = venue.Location.Latitude,
								Longitude = venue.Location.Longitude
							};
							p.BindingContext = venue;
							p.Clicked += Pin_Clicked;
							p.Color = SetPinColor(venue.Type);
							p.ZIndex = 0.3f;
							this.Map.CustomPins.Add(p);
						}
					}

					this.Map.UpdatePins();
				}
			}
		}

		public override async Task OnPageLoaded()
		{
			await base.OnPageLoaded();

			await this.viewModel.LoadData();

			if (Device.RuntimePlatform == Device.iOS && !this.initialZoomDone)
			{
				// v případě, že se piny načtou dřív, než je OnAppearing
				if (this.Map?.CustomPins?.Count > 0)
				{
					this.Map?.ZoomOnAll();
					this.initialZoomDone = true;
				}
			}

			if (await Framework.Device.Device.Instance.PermissionsManager.RequestLocationInUsePermission())
			{
				this.Map.IsShowingUser = true;
			}
		}


		private void Pin_Clicked(object sender, EventArgs e)
		{
			SafeGoToPage(new VenueDetailPage((sender as MapPin).BindingContext as Venue));
		}

		private void Map_ShowAll_Clicked(object sender, EventArgs e)
		{
			this.Map.ZoomOnAll();
		}
	}
}