﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Skeleton.Mobile.Framework.Controls.Forms.Pages;
using Skeleton.Mobile.Framework.Data.Manager;
using Skeleton.Mobile.Framework.Error.Log;
using Skeleton.Mobile.ODM.Backend.Enums;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using Color = Xamarin.Forms.Color;

namespace Skeleton.Mobile.ODM.Views
{
	[XamlCompilation(XamlCompilationOptions.Compile)]
	public partial class RootPage : SkeletonMasterDetailPage
	{
		public RootPage()
		{
			InitializeComponent();
			this.MenuPage.ItemSelected += Menu_ItemSelected;
			this.GoToPage((Page)Activator.CreateInstance(HomePageType));
		}

		protected override Type HomePageType => typeof(HomePage);

		public new void GoToPage(Page page)
		{
			try
			{
				RootNavigationPage = new SkeletonNavigationPage(page)
				{
					BarBackgroundColor = (Color)App.Current.Resources["PrimaryDarkColor"],
					BarTextColor = Color.White
				};
				this.Detail = RootNavigationPage;
			}
			catch (Exception ex)
			{
				ErrorLog.Instance.StartAddExceptionRecord(true, ex);
			}
		}

		private void Menu_ItemSelected(object sender, SelectedItemChangedEventArgs e)
		{
			var item = e.SelectedItem as MenuItemViewModel;

			if (item != null)
			{
				if (item.TargetType != null)
				{
					this.GoToPage((Page)Activator.CreateInstance(item.TargetType));
				}
				else if (item.Click != null)
				{
					item.Click.Invoke();
				}
				else
				{
					this.GoToPage((Page)Activator.CreateInstance(HomePageType, false));
				}
			}
			this.IsPresented = false;
		}

		public void MarkItemAsSelected(Type targetType)
		{
			var menuVM = (this.MenuPage.BindingContext as MenuViewModel);
			menuVM.SelectedItem = menuVM.MenuItems.SingleOrDefault(item => item.TargetType == targetType);

		}

		protected override void OnGoToHomePage()
		{
			//menuInitTask se neawaituje, jelikož jsme v overridnuté metodě z frameworku
			this.MarkItemAsSelected(HomePageType);
		}

		public void BackButtonPressedToHome()
		{
			var stack = RootPage.RootNavigationPage.Navigation.NavigationStack;
			if (stack.Count == 1)
			{
				this.MarkItemAsSelected(typeof(HomePage));
			}
		}

		public static void SetIgnoreBackButton(bool ignore)
		{
			NavigationPage.SetHasBackButton(App.Current.MainPage, !ignore);
		}

		public static List<MenuItemViewModel> GetMenuItems => GetCommonItems();

		private static List<MenuItemViewModel> GetCommonItems()
		{
			var menuItems = new List<MenuItemViewModel>();
			{
				menuItems.Add(new MenuItemViewModel
				{
					Title = "Menu_Homepage".LocalizeResx(),
					ItemType = MenuItemType.Homepage,
					FontFamily = "FontAwesome5FreeSolid",
					Icon = "\uf015",
					TargetType = typeof(HomePage),
					Selected = true
				});
				menuItems.Add(new MenuItemViewModel
				{
					Title = "ODMNavigationPage_Title".LocalizeResx(),
					ItemType = MenuItemType.NavigationPage,
					FontFamily = "FontAwesome5FreeSolid",
					Icon = "\uf279",
					TargetType = typeof(OdmNavigationPage),
					Selected = false
				});
				menuItems.Add(new MenuItemViewModel
				{
					Title = "Menu_AboutApp".LocalizeResx(),
					ItemType = MenuItemType.AboutApp,
					FontFamily = "FontAwesome5FreeSolid",
					Icon = "\uf129",
					TargetType = typeof(AboutAppPage),
					Selected = false
				});
				menuItems.Add(new MenuItemViewModel()
				{
					Title = "FoodPage_Title".LocalizeResx(),
					ItemType = MenuItemType.FoodMenuPage,
					FontFamily = "FontAwesome5FreeSolid",
					Icon = "\uf2e7",
					TargetType = typeof(FoodMenuRootPage),
					Selected = false,
				});
				menuItems.Add(new MenuItemViewModel()
				{
					Title = "SponsorsPage_Title".LocalizeResx(),
					ItemType = MenuItemType.FoodMenuPage,
					FontFamily = "FontAwesome5FreeSolid",
					Icon = "\uf2b5",
					TargetType = typeof(SponsorsPage),
					Selected = false,
				});
				menuItems.Add(new MenuItemViewModel
				{
					Title = "MainProgramRootPage_Title".LocalizeResx(),
					ItemType = MenuItemType.MainProgram,
					FontFamily = "FontAwesome5FreeSolid",
					Icon = "\uf073",
					TargetType = typeof(MainProgramRootPage),
					Selected = false
				});
			}

			menuItems = menuItems.OrderBy(item => (int)(item.ItemType)).ToList();
			return menuItems;

		}
	}
}