﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.ODM.Converters.IconConveters;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels;
using Skeleton.Mobile.ODM.Views.Abstract;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace Skeleton.Mobile.ODM.Views
{
	[XamlCompilation(XamlCompilationOptions.Compile)]
	public partial class VenueListPage : ODMPage
	{
		private VenueListViewModel viewModel;
		private ToolbarItem filterItem;
		public VenueListPage(List<Venue> venueList, bool filter)
		{
			InitializeComponent();
			this.viewModel = new VenueListViewModel(venueList);
			this.viewModel.VenueModels = venueList;
			this.viewModel.IconChange = IconChange_CallBack;
			this.BindingContext = this.viewModel;

			if (filter)
			{
				//< ToolbarItem x: Name = "FilterIcon" Text = "{extensions:Translate VenueList_Filter}" Icon = "{Binding IsFilterActive, Converter={StaticResource FilterIconConverter}}" Command = "{Binding FilterClickedCommand}" />
				filterItem = new ToolbarItem()
				{
					Text = "VenueList_Filter".LocalizeResx(),
					Command = viewModel.FilterClickedCommand,
					Icon="filter_icon.png"
				};
				this.ToolbarItems.Add(filterItem);
			}
			else
			{
				filterItem = new ToolbarItem()
				{
					Text = "PlacesSearchPage_ShowOnMap".LocalizeResx(),
					Icon = "map.png",
					Command = new Command(
						(() => {SafeGoToPage(new MapPage( viewModel.VenueModels, false ));})),
				};
				this.ToolbarItems.Add(filterItem);
			}
		}
		//Callback protože filtrování nemusí být na stránce povolené, tak se ikona mění takhle
		private void IconChange_CallBack(bool icon)
		{
			var conv = new FilterIconConverter();
			filterItem.Icon = (string)conv.Convert(icon, null, null, null);
		}

		public override async Task OnPageLoaded()
		{
			await base.OnPageLoaded();
			await this.viewModel.LoadData();
		}
	}
}