﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using ODM.Backend.Models.Models.Interfaces;
using ODMWeb.DL.Entities.Interfaces;
using ODMWeb.DL.ForeignModels;

namespace ODMWeb.DL.Entities
{
	[Table("Events")]
	public class EventEntity : BaseEntity, IDbItem, IEvent, ILocalDataLifecycle
	{
		public DateTime Start { get; set; }
		public DateTime? End { get; set; }

		public int VenueId { get; set; }
		[ForeignKey(nameof(VenueId))]
		public virtual VenueEntity Venue { get; set; }

		public int CompetitionId { get; set; }
		public string Title { get; set; }

		[ForeignKey(nameof(CompetitionId))]
		public virtual CompetitionEntity Competition { get; set; }

		/// <summary>
		/// Akce
		/// </summary>
		public string Info { get; set; }

		public EventEntity()
		{
			this.EventResults = new HashSet<EventResultEntity>();
			this.CompetingRegions = new HashSet<RegionEntity>();
		}

		public virtual ICollection<EventResultEntity> EventResults { get; set; }
		public virtual ICollection<RegionEntity> CompetingRegions { get; set; }
		public bool IsApproved { get; set; }

		public static bool ForeignEq(EventEntity rhs, ForeignEvent lhs)
		{
			return rhs.Start.Equals(lhs.Start) && Equals(rhs.End, lhs.End) && rhs.Title == lhs.Title &&
				   rhs.Info == lhs.Info;
		}
	}
}