﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using ODM.Backend.Models.Models.Interfaces;
using ODMWeb.DL.ForeignModels;

namespace ODMWeb.DL.Entities
{
	[Table("Venues")]
	public class VenueEntity : BaseEntity, IVenue
	{
		public VenueEntity(string name, VenueTypeEnum type, double latitude, double longitude, string street, string streetNumber, string city)
		{
			this.Name = name;
			this.Type = type;
			this.Location = new Gps(latitude, longitude);
			this.Address = new Address(street, streetNumber, city);
		}

		public VenueEntity()
		{
			this.Events = new HashSet<EventEntity>();
		}

		public virtual ICollection<EventEntity> Events { get; set; }

		public virtual Address Address { get; set; }
		public virtual Gps Location { get; set; }
		public string Name { get; set; }
		public VenueTypeEnum Type { get; set; }


		public static bool ForeignEq(VenueEntity rhs, ForeignVenue lhs)
		{
			return rhs.Name == lhs.Name
					&& rhs.Address.City == lhs.AddressCity
					&& rhs.Address.Street == lhs.AddressStreet
					&& rhs.Address.StreetNumber == lhs.AddressStreetNumber
					&& rhs.Location.Latitude == lhs.GpsLatitude
					&& rhs.Location.Longitude == lhs.GpsLongitude
					&& (int)rhs.Type == lhs.Type;
		}
	}
}