﻿using System;
using Newtonsoft.Json;

namespace ODMWeb.DL.ForeignModels
{
	internal interface IForeignObject
	{
		int ForeignId { get; set; }
	}

	public class ForeignSport : IForeignObject
	{
		public ForeignSport(int foreignId, string name)
		{
			ForeignId = foreignId;
			Name = name;
		}
		public int ForeignId { get; set; }

		public string Name { get; set; }
	}

	public class ForeignRegion : IForeignObject
	{
		public ForeignRegion(int foreignId, string name)
		{
			ForeignId = foreignId;
			Name = name;
		}

		public ForeignRegion()
		{
			
		}

		public int ForeignId { get; set; }

		public string Name { get; set; }
	}

	public class ForeignCompetition : IForeignObject
	{
		[JsonProperty(PropertyName = "Id")]
		public int ForeignId { get; set; }

		[JsonProperty(PropertyName = "SportId")]
		public int? ForeignSportId { get; set; }

		[JsonProperty(PropertyName = "Discipline")]
		public string Discipline { get; set; }
		[JsonProperty(PropertyName = "Race")]
		public string Race { get; set; }
	}

	public class ForeignCompetitionResult : IForeignObject
	{
		public int ForeignId { get; set; }

		public int ForeignCompetitionId { get; set; }
		public int? ForeignRegionId { get; set; }

		public string Competitior { get; set; }
		public string Result { get; set; }
		public int Rank { get; set; }
		public int Points { get; set; }
	}

	public class ForeignEvent : IForeignObject
	{
		[JsonProperty(PropertyName = "Id")]
		public int ForeignId { get; set; }

		[JsonProperty(PropertyName = "CompetitionId")]
		public int ForeignCompetitionId { get; set; }
		[JsonProperty(PropertyName = "VenueId")]
		public int ForeignVenueId { get; set; }

		[JsonProperty(PropertyName = "Start")]
		public DateTime Start { get; set; }
		[JsonProperty(PropertyName = "End")]
		public DateTime? End { get; set; }
		[JsonProperty(PropertyName = "Title")]
		public string Title { get; set; }
		[JsonProperty(PropertyName = "Info")]
		public string Info { get; set; }
	}

	public class ForeignEventResult : IForeignObject
	{
		public int ForeignId { get; set; }

		public int ForeignEventId { get; set; }
		public int ForeignRegionId { get; set; }

		public string Competitior { get; set; }
		public string Result { get; set; }
		public int Rank { get; set; }
		public string Points { get; set; }
	}

	public class ForeignVenue : IForeignObject
	{
		public ForeignVenue(int foreignId, string name, int type, double gpsLatitude, double gpsLongitude, string addressStreet, string addressStreetNumber, string addressCity)
		{
			ForeignId = foreignId;
			Name = name;
			Type = type;
			GpsLatitude = gpsLatitude;
			GpsLongitude = gpsLongitude;
			AddressStreet = addressStreet;
			AddressStreetNumber = addressStreetNumber;
			AddressCity = addressCity;
		}

		public ForeignVenue()
		{
			
		}

		public int ForeignId { get; set; }

		public string Name { get; set; }
		public int Type { get; set; }
		public double GpsLatitude { get; set; }
		public double GpsLongitude { get; set; }
		public string AddressStreet { get; set; }
		public string AddressStreetNumber { get; set; }
		public string AddressCity { get; set; }
	}
}