﻿using System;
using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using ODMWeb.DL.Entities;
using ODMWeb.DL.Entities.Serialization;

namespace ODMWeb.DL
{
	public class OdmDbContext : IdentityDbContext, IDisposable
	{

		public OdmDbContext(DbContextOptions options) : base(options)
		{
		}

		#region Static Data
		public virtual DbSet<VenueEntity> Venues { get; set; }
		public virtual DbSet<MealEntity> Meals { get; set; }
		public virtual DbSet<MenuEntity> Menus { get; set; }
		public virtual DbSet<RegionEntity> Regions { get; set; }
		public virtual DbSet<SportEntity> Sports { get; set; }
		#endregion

		#region Message
		public virtual DbSet<MessageEntity> Messages { get; set; }
		#endregion

		#region Program Data
		public DbSet<CompetitionEntity> Competitions { get; set; }
		public DbSet<EventEntity> Events { get; set; }
		#endregion

		#region Result Data
		public DbSet<CompetitionResultEntity> CompetitionResults { get; set; }
		public DbSet<EventResultEntity> EventResults { get; set; }
		#endregion

		#region Management

		public DbSet<SerializationRecordEntity> SerializedFiles { get; set; }

		#endregion

		protected override void OnModelCreating(ModelBuilder modelBuilder)
		{
			base.OnModelCreating(modelBuilder);

			modelBuilder.Entity<CompetitionEntity>().HasMany(c => c.Events).WithOne(e => e.Competition)
				.OnDelete(DeleteBehavior.Cascade);
			modelBuilder.Entity<CompetitionEntity>().HasMany(c => c.CompetitionResults).WithOne(r => r.Competition)
				.OnDelete(DeleteBehavior.Cascade);
			modelBuilder.Entity<EventEntity>().HasMany(e => e.EventResults).WithOne(r => r.Event)
				.OnDelete(DeleteBehavior.Cascade);
			modelBuilder.Entity<MenuEntity>().HasMany(m => m.Meals).WithOne(m => m.Menu)
				.OnDelete(DeleteBehavior.Cascade);

			modelBuilder.Entity<SportEntity>().OwnsOne(s => s.Item,
				i =>
				{
					i.Property(p => p.Key).HasColumnName("Sport_Key");
					i.Property(p => p.Name).HasColumnName("Sport_Name");
				});

			modelBuilder.Entity<VenueEntity>().OwnsOne(v => v.Address,
				a =>
				{
					a.Property(p => p.City).HasColumnName("Venue_Address_City");
					a.Property(p => p.Street).HasColumnName("Venue_Address_Street");
					a.Property(p => p.StreetNumber).HasColumnName("Venue_Address_StreetNumber");
				});

			modelBuilder.Entity<VenueEntity>().OwnsOne(v => v.Location,
				l =>
				{
					l.Property(p => p.Latitude).HasColumnName("Venue_Location_Latitude");
					l.Property(p => p.Longitude).HasColumnName("Venue_Location_Longitude");
				});
		}
	}
}