﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Enums;
using ODMWeb.DL.Entities;
using ODMWeb.DL.EntityDependencies;
using ODMWeb.DL.Exceptions;
using ODMWeb.DL.ForeignModels;
using ODMWeb.DL.Repositories.Interfaces;
using ODMWebService.DL.Entities.Enums;

namespace ODMWeb.DL.Repositories
{
	internal class CompetitionRepository : Repository<CompetitionEntity>, ICompetitionRepository<CompetitionDependencies>
	{
		public CompetitionRepository(OdmDbContext dbContext) : base(dbContext)
		{
		}

		public async Task<CompetitionEntity> GetCeremonial()
		{
			return await base.dbContext.Competitions.FirstOrDefaultAsync(c =>
				c.CompetitionType == CompetitionTypeEnum.Ceremonial);
		}

		public async Task<CompetitionEntity> GetEntertainment()
		{
			return await base.dbContext.Competitions.FirstOrDefaultAsync(c =>
				c.CompetitionType == CompetitionTypeEnum.Entertainment);
		}
		
		public async Task<int> UpdateFromForeign(List<ForeignCompetition> foreignCompetitions)
		{

			foreach (var comp in foreignCompetitions)
			{
				var dependencies = await UpdateSingleFromForeign(comp);
				if (!dependencies.Result)
				{
					throw new SportDependencyNotResolvedException(dependencies.ForeignSportId.ToString());
				}
			}

			return
				await dbContext.SaveChangesAsync();
		}

		public async Task<CompetitionDependencies> UpdateSingleFromForeign(ForeignCompetition foreignItem)
		{
			var dependencies = new CompetitionDependencies(){Result = true};
			if (foreignItem.ForeignSportId == null)
			{
				return dependencies;
			}
			var sport = await base.dbContext.Sports.FirstOrDefaultAsync(s => s.ForeignApiId == foreignItem.ForeignSportId);

			if (sport is null)
			{
				dependencies.ForeignSportId = foreignItem.ForeignSportId;
				dependencies.Result = false;
				return dependencies;
			}

			var entity = await base.dbContext.Competitions.FirstOrDefaultAsync(c => c.ForeignApiId == foreignItem.ForeignId);

			if (entity is null)
			{
				sport.Competitions.Add(new CompetitionEntity()
				{
					ForeignApiId = foreignItem.ForeignId,
					CompetitionType = CompetitionTypeEnum.Main,
					Discipline = foreignItem.Discipline,
					Race = foreignItem.Race,
					Gender = Gender.Unspecified,
				});
			}
			else
			{
				if (entity.Sport != sport)
				{
					entity.Sport = sport;
				}

				if (CompetitionEntity.ForeignEq(entity, foreignItem)) return dependencies;
				entity.Discipline = foreignItem.Discipline;
				entity.Race = foreignItem.Race;
				entity.Gender = Gender.Unspecified;
			}

			return dependencies;
		}
	}
}
