﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using ODMWeb.DL.Entities;
using ODMWeb.DL.EntityDependencies;
using ODMWeb.DL.Exceptions;
using ODMWeb.DL.ForeignModels;
using ODMWeb.DL.Repositories.Interfaces;

namespace ODMWeb.DL.Repositories
{
	internal class EventRepository : Repository<EventEntity>, IEventRepository<EventDependencies>
	{
		public EventRepository(OdmDbContext dbContext) : base(dbContext)
		{
		}

		public async Task<int> UpdateFromForeign(List<ForeignEvent> foreignList)
		{
			foreach (var fEvent in foreignList)
			{
				var dependencies = await UpdateSingleFromForeign(fEvent);
				if (!dependencies.Result)
				{
					var compException = new CompetitionDependencyNotResolvedException(dependencies.ForeignCompetitionId.ToString());

					var venuException = new VenueDependencyNotResolvedException(dependencies.ForeignVenueId.ToString());

					if (dependencies.ForeignCompetitionId != null && dependencies.ForeignVenueId != null)
					{
						throw new AggregateException(compException, venuException);
					}
					else if (dependencies.ForeignCompetitionId != null)
					{
						throw compException;
					}
					else if (dependencies.ForeignVenueId != null)
					{
						throw venuException;
					}
				}
			}

			return
				await dbContext.SaveChangesAsync();
		}

		public async Task<EventDependencies> UpdateSingleFromForeign(ForeignEvent foreignItem)
		{
			var dependencies = new EventDependencies() { Result = true };

			var competiton = await dbContext.Competitions.FirstOrDefaultAsync(c => c.ForeignApiId == foreignItem.ForeignCompetitionId);
			if (competiton is null)
			{
				dependencies.Result = false;
				dependencies.ForeignCompetitionId = foreignItem.ForeignCompetitionId;
			}
			var venue = await dbContext.Venues.FirstOrDefaultAsync(v => v.ForeignApiId == foreignItem.ForeignVenueId);
			if (venue is null)
			{
				dependencies.Result = false;
				dependencies.ForeignVenueId = foreignItem.ForeignVenueId;
			}

			if (!dependencies.Result) return dependencies;

			var entity = await dbContext.Events.FirstOrDefaultAsync(e => e.ForeignApiId == foreignItem.ForeignId);
			if (entity is null)
			{
				competiton.Events.Add(new EventEntity()
				{
					ForeignApiId = foreignItem.ForeignId,
					Start = foreignItem.Start,
					End = foreignItem.End,
					Title = foreignItem.Title,
					Info = foreignItem.Info,
					Venue = venue,
					VenueId = venue.Id,
				});
			}
			else
			{
				if (entity.Venue != venue)
				{
					entity.Venue = venue;
				}

				if (entity.Competition != competiton)
				{
					entity.Competition = competiton;
				}

				if (EventEntity.ForeignEq(entity, foreignItem)) return dependencies;
				entity.Start = foreignItem.Start;
				entity.End = foreignItem.End;
				entity.Title = foreignItem.Title;
				entity.Info = foreignItem.Info;
			}

			return dependencies;
		}
	}
}