﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using ODMWeb.DL.Entities;
using ODMWeb.DL.ForeignModels;
using ODMWeb.DL.Repositories.Interfaces;

namespace ODMWeb.DL.Repositories
{
	internal class RegionRepository : Repository<RegionEntity>, IRegionRepository
	{
		public RegionRepository(OdmDbContext dbContext) : base(dbContext)
		{
		}

		public async Task<int> UpdateFromForeign(List<ForeignRegion> foreignList)
		{
			foreach (var region in foreignList)
			{
				await UpdateSingleFromForeign(region);
			}

			return
				await dbContext.SaveChangesAsync();
		}

		public async Task<BaseDependencies> UpdateSingleFromForeign(ForeignRegion foreignItem)
		{
			var entity = await dbContext.Regions.FirstOrDefaultAsync(r => r.ForeignApiId == foreignItem.ForeignId);

			if (entity is null)
			{
				dbContext.Regions.Add(new RegionEntity()
				{
					Name = foreignItem.Name,
					ForeignApiId = foreignItem.ForeignId
				});
			}
			else
			{
				if (RegionEntity.ForeignEq(entity, foreignItem)) return new BaseDependencies(){Result = true};
				entity.Name = foreignItem.Name;
			}

			return new BaseDependencies(){Result = true};
		}
	}
}