﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using ODMWeb.DL.Entities;
using ODMWeb.DL.Repositories.Interfaces;

namespace ODMWeb.DL.Repositories
{
	internal class Repository<TEntity> : IRepository<TEntity> where TEntity : BaseEntity
	{
		protected readonly OdmDbContext dbContext;

		public Repository(OdmDbContext dbContext)
		{
			this.dbContext = dbContext;
		}

		public Task<TEntity> GetById(int id)
		{
			return this.dbContext.Set<TEntity>().FindAsync(id);
		}

		public void Add(TEntity entity)
		{
			this.dbContext.Set<TEntity>().Add(entity);
		}

		public void AddRange(IEnumerable<TEntity> entities)
		{
			this.dbContext.Set<TEntity>().AddRange(entities);
		}

		public async Task Remove(TEntity entity)
		{
			var foundEntity = await this.dbContext.Set<TEntity>().FindAsync(entity.Id);
			foundEntity.IsRemoved = true;
		}

		public async Task RemoveRange(IEnumerable<TEntity> entities)
		{
			foreach (var entity in entities)
			{
				await Remove(entity);
			}
		}

		public TEntity FirstOrDefault()
		{
			return this.dbContext.Set<TEntity>().FirstOrDefault();
		}

		public virtual IQueryable<TEntity> GetAll()
		{
			IQueryable<TEntity> query = this.dbContext.Set<TEntity>();
			return query;
		}

		public IQueryable<TEntity> Find(Expression<Func<TEntity, bool>> predicate)
		{
			IQueryable<TEntity> query = this.dbContext.Set<TEntity>().Where(predicate);
			return query;
		}

		public virtual void Update(TEntity entity)
		{
			this.dbContext.Set<TEntity>().Attach(entity);
			this.dbContext.Entry(entity).State = EntityState.Modified;
			var updateEntity = this.dbContext.Set<TEntity>().Find(entity.Id);
			updateEntity.IsModified = true;
			this.dbContext.SaveChanges();
		}
	}
} 