﻿using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Enums;
using ODMWeb.DL.Entities;
using ODMWeb.DL.ForeignModels;
using ODMWeb.DL.Repositories.Interfaces;

namespace ODMWeb.DL.Repositories
{
	internal class VenueRepository : Repository<VenueEntity>, IVenueRepository
	{
		public VenueRepository(OdmDbContext dbContext) : base(dbContext)
		{
		}

		public async Task<int> UpdateFromForeign(List<ForeignVenue> foreignList)
		{
			foreach (var venue in foreignList)
			{
				await UpdateSingleFromForeign(venue);
			}

			return await dbContext.SaveChangesAsync();
		}

		public async Task<BaseDependencies> UpdateSingleFromForeign(ForeignVenue foreignItem)
		{
			var entity = await dbContext.Venues.FirstOrDefaultAsync(v => v.ForeignApiId == foreignItem.ForeignId);
			if (entity is null)
			{
				dbContext.Venues.Add(new VenueEntity(foreignItem.Name, (VenueTypeEnum)foreignItem.Type, foreignItem.GpsLatitude,
					foreignItem.GpsLongitude, foreignItem.AddressStreet, foreignItem.AddressStreetNumber, foreignItem.AddressCity)
				{
					ForeignApiId = foreignItem.ForeignId
				});
			}
			else
			{
				if (VenueEntity.ForeignEq(entity, foreignItem)) return new BaseDependencies(){Result = true};
				entity.Name = foreignItem.Name;
				entity.Type = (VenueTypeEnum)foreignItem.Type;
				entity.Address.City = foreignItem.AddressCity;
				entity.Address.StreetNumber = foreignItem.AddressStreetNumber;
				entity.Address.Street = foreignItem.AddressStreet;
				entity.Location.Latitude = foreignItem.GpsLatitude;
				entity.Location.Longitude = foreignItem.GpsLongitude;
			}

			return new BaseDependencies(){Result = true};
		}
	}
}