﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using ODMWeb.DL.Entities;
using ODMWebService.DL.Entities.Enums;

namespace ODMWeb.DL.Seed
{
	public static class DatabaseInitializer
	{
		public static void Seed(UserManager<IdentityUser> userManager, RoleManager<IdentityRole> roleManager, OdmDbContext dbContext)
		{
			// Vytvoření administračního účtu a uživatelských rolí.
			IdentityInitializer.SeedData(userManager, roleManager);

			if (dbContext.Competitions.FirstOrDefault(c => c.CompetitionType == CompetitionTypeEnum.Ceremonial) is null)
			{
				dbContext.Competitions.Add(new CompetitionEntity()
				{
					CompetitionType = CompetitionTypeEnum.Ceremonial,
					Discipline =  "Ceremoniály",
					Gender = Gender.Unspecified,
					Race = "",
				});
			}

			if (dbContext.Competitions.FirstOrDefault(c => c.CompetitionType == CompetitionTypeEnum.Entertainment) is null)
			{
				dbContext.Competitions.Add(new CompetitionEntity()
				{
					CompetitionType = CompetitionTypeEnum.Entertainment,
					Discipline = "Doprovodný program",
					Gender = Gender.Unspecified,
					Race = "",
				});
			}

			// Uložení změn do databáze.
			dbContext.SaveChanges();
		}
	}

	internal static class IdentityInitializer
	{
		public static void SeedData(UserManager<IdentityUser> userManager, RoleManager<IdentityRole> roleManager)
		{
			SeedRoles(roleManager);
			SeedUsers(userManager);
		}

		private static readonly string[] Roles =
		{
			"BenevolentDictatorForLife",
			"Overseer"
		};

		private static readonly string[][] Users =
		{
			new []{"admin", "Admin123!", "BenevolentDictatorForLife"},
			new []{"overseer", "Admin123!", "Overseer"},
		};

		public static void SeedUsers(UserManager<IdentityUser> userManager)
		{
			foreach (var userDef in Users)
			{
				if (userManager.FindByNameAsync(userDef[0]).Result == null)
				{
					var user = new IdentityUser();
					user.UserName = userDef[0];

					IdentityResult result = userManager.CreateAsync(user, userDef[1]).Result;

					if (result.Succeeded)
					{
						userManager.AddToRoleAsync(user, userDef[2]).Wait();
					}
				}
			}
		}

		public static void SeedRoles(RoleManager<IdentityRole> roleManager)
		{
			foreach (var roleString in Roles)
			{
				if (!roleManager.RoleExistsAsync(roleString).Result)
				{
					var role = new IdentityRole(roleString);
					role.Name = roleString;
					IdentityResult roleResult = roleManager.
						CreateAsync(role).Result;
				}
			}
		}
	}
}