﻿using System;
using System.Threading.Tasks;
using ODMWeb.DL.Entities;
using ODMWeb.DL.EntityDependencies;
using ODMWeb.DL.Repositories;
using ODMWeb.DL.Repositories.Interfaces;

namespace ODMWeb.DL
{
	public class UnitOfWork : IUnitOfWork, IDisposable
	{
		private readonly OdmDbContext context;

		public IVenueRepository Venues { get; }
		public IRepository<MealEntity> Meals { get; }
		public IRepository<MenuEntity> Menus { get; }
		public IRegionRepository Regions { get; }
		public ISportRepository Sports { get; }
		public IRepository<MessageEntity> Messages { get; }
		public ICompetitionResultRepository CompetitionResults { get; }
		public IEventResultRepository EventResults { get; }
		public SerializationRecordRepository SerializedFiles { get; set; }
		public ICompetitionRepository<CompetitionDependencies> Competitions { get; set; }
		public IEventRepository<EventDependencies> Events { get; set; }

		public UnitOfWork(OdmDbContext context)
		{
			this.context = context;
			this.Venues = new VenueRepository(this.context);
			this.Meals = new Repository<MealEntity>(this.context);
			this.Menus = new Repository<MenuEntity>(this.context);
			this.Regions = new RegionRepository(this.context);
			this.Sports = new SportRepository(this.context);
			this.Messages = new Repository<MessageEntity>(this.context);
			this.Competitions = new CompetitionRepository(this.context);
			this.Events = new EventRepository(this.context);
			this.CompetitionResults = new CompetitionResultRepository(this.context);
			this.EventResults = new EventResultRepository(this.context);
			this.SerializedFiles = new SerializationRecordRepository(this.context);
		}

		public async Task<int> Save()
		{
			var result = await this.context.SaveChangesAsync();
			return result;
		}

		public void Dispose()
		{
			context?.Dispose();
		}
	}
}