﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Claims;
using System.Threading;
using System.Threading.Tasks;
using AutoMapper;
using Hangfire;
using Hangfire.Storage;
using Hangfire.Storage.Monitoring;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using ODMWeb.DataProviding;
using ODMWeb.DataProviding.Publishers;
using ODMWeb.DataProviding.Serialization;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;
using Serilog;
using ILogger = Serilog.ILogger;

namespace ODMWeb.Controllers.Api
{
	[Route("api/stage")]
	[ApiController]
	[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
	public class PublishOdmDataController : ControllerBase
	{
		private async Task<bool> CheckJobState(string taskId)
		{
			Task checkJobState = Task.Factory.StartNew(() =>
				{
					while (true)
					{
						IMonitoringApi monitoringApi = JobStorage.Current.GetMonitoringApi();
						JobDetailsDto jobDetailsDto = monitoringApi.JobDetails(taskId);
						string currentState = jobDetailsDto.History[0].StateName;
						if (currentState != "Enqueued" && currentState != "Processing")
						{
							break;
						}
						Thread.Sleep(100);
					}
				});
			await checkJobState;
			return checkJobState.IsCompletedSuccessfully;
		}

		[HttpGet("static")]
		public async Task<bool> PublishStaticData()
		{
			var job = BackgroundJob.Enqueue<StaticDataPublisher>(p => p.Run(User.Identity.Name));
			return await CheckJobState(job);
		}

		[HttpGet("message")]
		public async Task<bool> PublishMessageData()
		{
			var job = BackgroundJob.Enqueue<MessageDataPublisher>(p => p.Run(User.Identity.Name));
			return await CheckJobState(job);
		}

		[HttpGet("program")]
		public async Task<bool> PublishProgramData()
		{
			var job = BackgroundJob.Enqueue<ProgramDataPublisher>(p => p.Run(User.Identity.Name));
			return await CheckJobState(job);
		}

		[HttpGet("result")]
		public async Task<bool> PublishResultData()
		{
			var job = BackgroundJob.Enqueue<ResultDataPublisher>(p => p.Run(User.Identity.Name));
			return await CheckJobState(job);
		}
	}
}