﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;
using ODMWeb.Models.Home;
using Serilog;

namespace ODMWeb.Controllers
{
	public class HomeController : Controller
	{
		private readonly IUnitOfWork unitOfWork;
		private readonly IMapper mapper;

		public HomeController(IUnitOfWork unitOfWork, IMapper mapper)
		{
			this.unitOfWork = unitOfWork;
			this.mapper = mapper;
		}

		public async Task<IActionResult> Index()
		{
			if (!User.Identity.IsAuthenticated)
			{
				return View();
			}

			var viewModel = new DashboardViewModel
			{
				MessageFile = mapper.Map<SerializationRecordItem>(
					await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Message)) ?? new SerializationRecordItem(){Type = SerializationRecordType.Message},
				StaticFile = mapper.Map<SerializationRecordItem>(
					await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Static)) ?? new SerializationRecordItem(){Type = SerializationRecordType.Static},
				ProgramFile = mapper.Map<SerializationRecordItem>(
					await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Program)) ?? new SerializationRecordItem(){Type = SerializationRecordType.Program},
				ResultFile = mapper.Map<SerializationRecordItem>(
					await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Result)) ?? new SerializationRecordItem(){Type = SerializationRecordType.Result}
			};

			return View("Dashboard", viewModel);
		}

		[Authorize]
		[HttpGet("[controller]/[action]/{type}/{version}")]
		public IActionResult Exact(SerializationRecordType type, int version)
		{
			if (type == SerializationRecordType.Static)
				return RedirectToAction("StaticExact", "OdmData", new {version});
			if (type == SerializationRecordType.Message)
				return RedirectToAction("MessageExact", "OdmData", new {version});
			if (type == SerializationRecordType.Program)
				return RedirectToAction("ProgramExact", "OdmData", new {version});
			if (type == SerializationRecordType.Result) 
				return RedirectToAction("ResultExact", "OdmData", new {version});

			return RedirectToAction("Index");
		}
	}
}