﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;
using Serilog;

namespace ODMWeb.DataProviding.ApiProviders
{
	public class MessageApiProvider : IApiProvider<MessageAppData>
	{
		private readonly ILogger logger = Log.ForContext<MessageApiProvider>();

		private IUnitOfWork unitOfWork;
		public MessageApiProvider(IUnitOfWork unitOfWork)
		{
			this.unitOfWork = unitOfWork;
		}

		public async Task<MessageAppData> GetLastAvailableData(SerializationRecordEntity record)
		{
			MessageAppData output = null;
			SerializationRecordEntity lastRecord = record;

			while (output == null && lastRecord != null)
			{
				output = await GetData(lastRecord);

				if (output == null)
				{
					lastRecord =
						await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Message);
				}
			}

			return output;
		}

		public async Task<MessageAppData> GetData(SerializationRecordEntity record)
		{
			MessageAppData output = null;

			if (record == null || !File.Exists(record.FilePath))
			{
				return output;
			}
			try
			{
				using (var reader = new StreamReader(record.FilePath))
				{
					var serializer = new XmlSerializer(typeof(MessageAppData));

					output = (MessageAppData)serializer.Deserialize(reader.BaseStream);
				}
			}
			catch (Exception ex)
			{
				output = null;
				logger.Error(ex, "GetData encountered an exception.");
				await unitOfWork.SerializedFiles.SetIsAvailable(record, false);
				logger.Warning("Record no longer available: {@SerializationRecordEntity}", record);
			}

			return output;
		}

		public MessageAppData GetChanges(MessageAppData client, MessageAppData server)
		{
			MessageAppData output = new MessageAppData() { Version = server.Version };

			output.Messages = server.Messages.Except(client.Messages, new MessageComparer()).ToList();

			return output;
		}
	}
}