﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;
using Serilog;

namespace ODMWeb.DataProviding.ApiProviders
{
	public class ProgramApiProvider : IApiProvider<ProgramAppData>
	{
		private readonly ILogger logger = Log.ForContext<ProgramApiProvider>();
		private IUnitOfWork unitOfWork;
		public ProgramApiProvider(IUnitOfWork unitOfWork)
		{
			this.unitOfWork = unitOfWork;
		}

		public ProgramAppData GetChanges(ProgramAppData client, ProgramAppData server)
		{
			ProgramAppData output = new ProgramAppData() { Version = server.Version };

			output.Competitions = server.Competitions.Except(client.Competitions, new CompetitionComparer()).ToList();
			output.Events = server.Events.Except(client.Events, new EventComparer()).ToList();

			return output;
		}

		public async Task<ProgramAppData> GetData(SerializationRecordEntity record)
		{
			ProgramAppData output = null;

			if (record == null || !File.Exists(record.FilePath))
			{
				return output;
			}

			try
			{
				using (var reader = new StreamReader(record.FilePath))
				{
					var serializer = new XmlSerializer(typeof(ProgramAppData));

					output = (ProgramAppData)serializer.Deserialize(reader.BaseStream);
				}
			}
			catch (Exception ex)
			{
				output = null; 
				logger.Error(ex, "GetData encountered an exception.");
				await unitOfWork.SerializedFiles.SetIsAvailable(record, false);
				logger.Warning("Record no longer available: {@SerializationRecordEntity}", record);
			}

			return output;
		}

		public async Task<ProgramAppData> GetLastAvailableData(SerializationRecordEntity record)
		{
			ProgramAppData output = null;
			SerializationRecordEntity lastRecord = record;

			while (output == null && lastRecord != null)
			{
				output = await GetData(lastRecord);

				if (output == null)
				{
					lastRecord =
						await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Program);
				}
			}

			return output;
		}
	}
}