﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;
using Serilog;

namespace ODMWeb.DataProviding.ApiProviders
{
	public class ResultApiProvider : IApiProvider<ResultAppData>
	{
		private readonly ILogger logger = Log.ForContext<ResultApiProvider>();
		private IUnitOfWork unitOfWork;
		public ResultApiProvider(IUnitOfWork unitOfWork)
		{
			this.unitOfWork = unitOfWork;
		}

		public ResultAppData GetChanges(ResultAppData client, ResultAppData server)
		{
			ResultAppData output = new ResultAppData() { Version = server.Version };

			output.CompetitionResults = server.CompetitionResults.Except(client.CompetitionResults, new CompetitionResultComparer()).ToList();
			output.EventResults = server.EventResults.Except(client.EventResults, new EventResultComparer()).ToList();

			return output;
		}

		public async Task<ResultAppData> GetData(SerializationRecordEntity record)
		{
			ResultAppData output = null;

			if (record == null || !File.Exists(record.FilePath))
			{
				return output;
			}

			try
			{
				using (var reader = new StreamReader(record.FilePath))
				{
					var serializer = new XmlSerializer(typeof(ResultAppData));

					output = (ResultAppData)serializer.Deserialize(reader.BaseStream);
				}
			}
			catch (Exception ex)
			{
				output = null; 
				logger.Error(ex, "GetData encountered an exception.");
				await unitOfWork.SerializedFiles.SetIsAvailable(record, false);
				logger.Warning("Record no longer available: {@SerializationRecordEntity}", record);
			}

			return output;
		}

		public async Task<ResultAppData> GetLastAvailableData(SerializationRecordEntity record)
		{
			ResultAppData output = null;
			SerializationRecordEntity lastRecord = record;

			while (output == null && lastRecord != null)
			{
				output = await GetData(lastRecord);

				if (output == null)
				{
					lastRecord =
						await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Result);
				}
			}

			return output;
		}
	}
}