﻿using System.IO;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Hosting;
using ODMWeb.DataProviding.Serialization;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;

namespace ODMWeb.DataProviding.Publishers
{
	public class ProgramDataPublisher : AbsDataPublisher
	{
		public ProgramDataPublisher(IHostingEnvironment environment, IMapper mapper, IUnitOfWork unitOfWork) : base(environment, mapper, unitOfWork)
		{
		}

		public override async Task<bool> Run(string userName)
		{
			var serializer = new ProgramDataSerializer();
			var entityData = await serializer.CreateEntityGroup(this.UnitOfWork);
			var modelData = serializer.CreateDataGroup(this.Mapper, entityData);
			var version = await UnitOfWork.SerializedFiles.GetLastVersionByType(SerializationRecordType.Program) + 1;
			var path = Path.Combine(FolderPath, Config.GetProgramVersionFileName(version));

			var record = await UnitOfWork.SerializedFiles.Add(new SerializationRecordEntity(version, SerializationRecordType.Program, path, userName));

			bool result = serializer.SerializeData(modelData, path, version, UnitOfWork);
			if (result)
			{
				await UnitOfWork.SerializedFiles.SetIsAvailable(record, true);
				await serializer.SetEntitiesPublished(entityData, UnitOfWork);

				Logger.Warning("PublishProgramData successful by user: {Name}. Record: {@Record}",
					userName, record);
			}
			else
			{
				Logger.Error("PublishProgramData failed by user: {Name}. Record: {@Record}", userName, record);
			}
			return result;
		}
	}
}