﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using AutoMapper;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities;
using ODMWeb.DL.Entities.Serialization.Groups;
using Serilog;

namespace ODMWeb.DataProviding.Serialization
{
	public class MessageDataSerializer : IDataSerializer<MessageEntityData, MessageAppData>
	{
		private readonly ILogger logger = Log.ForContext<MessageDataSerializer>();

		public async Task<MessageEntityData> CreateEntityGroup(IUnitOfWork unitOfWork)
		{
			MessageEntityData output = new MessageEntityData();
			var messages = await unitOfWork.Messages.GetAll()
				//zprávy které jsou schválené
				.Where(e => e.IsApproved).ToListAsync();

			output.Messages = messages.ToList();

			return output;
		}

		public MessageAppData CreateDataGroup(IMapper mapper, MessageEntityData entities)
		{
			MessageAppData output;
			output = mapper.Map<MessageAppData>(entities);
			return output;
		}

		public bool SerializeData(MessageAppData data, string path, int version, IUnitOfWork unitOfWork = null)
		{
			var serializer = new XmlSerializer(typeof(MessageAppData));
			data.Version = version;

			using (var writer = new StreamWriter(path))
			{
				try
				{
					serializer.Serialize(writer, data);
				}
				catch (Exception ex)
				{
					logger.Error(ex, "Serializer encountered an exception.");
					return false;
				}
			}

			return true;
		}

		public async Task SetEntitiesPublished(MessageEntityData entities, IUnitOfWork unitOfWork)
		{
			foreach (var entity in entities.Messages)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			await unitOfWork.Save();
		}
	}
}