﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using AutoMapper;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities;
using ODMWeb.DL.Entities.Serialization.Groups;
using ODMWebService.DL.Entities.Enums;
using Serilog;

namespace ODMWeb.DataProviding.Serialization
{
	public class ProgramDataSerializer : IDataSerializer<ProgramEntityData, ProgramAppData>
	{
		private readonly ILogger logger = Log.ForContext<ProgramDataSerializer>();

		public ProgramAppData CreateDataGroup(IMapper mapper, ProgramEntityData entities)
		{
			ProgramAppData output;

			output = mapper.Map<ProgramAppData>(entities);

			return output;
		}

		public async Task<ProgramEntityData> CreateEntityGroup(IUnitOfWork unitOfWork)
		{
			ProgramEntityData output = new ProgramEntityData
			{
				Events = new List<EventEntity>(),
				Competitions = await unitOfWork.Competitions.GetAll()
					.Where(e => (e.IsSerialized == false && e.IsRemoved == false) || e.IsSerialized).ToListAsync()
			};

			foreach (var outputCompetition in output.Competitions)
			{
				if (outputCompetition.CompetitionType != CompetitionTypeEnum.Main)
				{
					output.Events.AddRange(outputCompetition.Events.Where(e => e.IsApproved));
				}
				else
				{
					output.Events.AddRange(outputCompetition.Events);
				}
			}

			return output;
		}

		public bool SerializeData(ProgramAppData data, string path, int version, IUnitOfWork unitOfWork)
		{
			var venues = unitOfWork.Venues.GetAll().ToList();
			foreach (var competition in data.Competitions)
			{
				var events = data.Events.Where(e => e.CompetitionId == competition.Id);
				foreach (var @event in events)
				{
					var venue = venues.Find(v => v.Id == @event.VenueId);

					if (competition.Cities is null)
					{
						competition.Cities = new List<string>();
					}

					if (!competition.Cities.Contains(venue.Address.City))
					{
						competition.Cities.Add(venue.Address.City);
					}
				}
			}

			var serializer = new XmlSerializer(typeof(ProgramAppData));
			data.Version = version;

			using (var writer = new StreamWriter(path))
			{
				try
				{
					serializer.Serialize(writer, data);
				}
				catch (Exception ex)
				{
					logger.Error(ex, "Serializer encountered an exception.");
					return false;
				}
			}

			return true;
		}

		public async Task SetEntitiesPublished(ProgramEntityData entities, IUnitOfWork unitOfWork)
		{
			foreach (var entity in entities.Competitions)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			foreach (var entity in entities.Events)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			await unitOfWork.Save();
		}
	}
}