﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using AutoMapper;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization.Groups;
using Serilog;
using ILogger = Serilog.ILogger;

namespace ODMWeb.DataProviding.Serialization
{
	public class ResultDataSerializer : IDataSerializer<ResultEntityData, ResultAppData>
	{
		private readonly ILogger logger = Log.ForContext<ResultDataSerializer>();

		public ResultAppData CreateDataGroup(IMapper mapper, ResultEntityData entities)
		{
			ResultAppData output;
			output = mapper.Map<ResultAppData>(entities);
			return output;
		}

		public async Task<ResultEntityData> CreateEntityGroup(IUnitOfWork unitOfWork)
		{
			ResultEntityData output = new ResultEntityData
			{
				CompetitionResults = await unitOfWork.CompetitionResults.GetAll()
					.Where(e => (e.IsSerialized == false && e.IsRemoved == false) || e.IsSerialized).ToListAsync(),
				EventResults = await unitOfWork.EventResults.GetAll()
					.Where(e => (e.IsSerialized == false && e.IsRemoved == false) || e.IsSerialized).ToListAsync()
			};

			return output;
		}

		public bool SerializeData(ResultAppData data, string path, int version, IUnitOfWork unitOfWork = null)
		{
			var serializer = new XmlSerializer(typeof(ResultAppData));
			data.Version = version;

			using (var writer = new StreamWriter(path))
			{
				try
				{
					serializer.Serialize(writer, data);
				}
				catch (Exception ex)
				{
					logger.Error(ex, "Serializer encountered an exception.");
					return false;
				}
			}

			return true;
		}

		public async Task SetEntitiesPublished(ResultEntityData entities, IUnitOfWork unitOfWork)
		{
			foreach (var entity in entities.EventResults)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			foreach (var entity in entities.CompetitionResults)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			await unitOfWork.Save();
		}
	}
}