﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using Newtonsoft.Json;
using ODMWeb.DataProviding.ApiProviders;
using ODMWeb.DL.Entities;
using ODMWeb.DL.ForeignModels;
using Serilog;

namespace ODMWeb.ForeignApiManager
{
	public class WebService
	{
		private readonly ILogger logger = Log.ForContext<WebService>();

		private async Task<string> RequestJson(string url)
		{
			string output;
			using (HttpClient client = new HttpClient())
			{
				try
				{
					logger.Information("Attempting GET request from: {Url}.", url);
					HttpResponseMessage response = await client.GetAsync(url);
					response.EnsureSuccessStatusCode();
					output = await response.Content.ReadAsStringAsync();
					logger.Information("GET request from: {Url} successful.", url);
				}
				catch (Exception e)
				{
					output = "";
					logger.Error(e, "Error happened during GET request from: {Url}.", url);
				}
			}

			return output;
		}

		public async Task<List<ForeignCompetition>> GetCompetitions()
		{
			var jsonResponse = await this.RequestJson(Configuration.CompetitionsAction);
			List<ForeignCompetition> output;
			try
			{
				output = JsonConvert.DeserializeObject<List<ForeignCompetition>>(jsonResponse);
			}
			catch (Exception e)
			{
				output = null;
				logger.Error(e, "Error parsing foreign Competition data.");
			}

			return output;
		}

		public async Task<List<ForeignEvent>> GetEvents()
		{
			var jsonResponse = await this.RequestJson(Configuration.EventsAction);

			List<ForeignEvent> output;
			try
			{
				output = JsonConvert.DeserializeObject<List<ForeignEvent>>(jsonResponse);
			}
			catch (Exception e)
			{
				output = null;
				logger.Error(e, "Error parsing foreign Event data.");
			}

			return output;
		}

#warning TEST METHOD
		public async Task<List<ForeignRegion>> GetRegions()
		{
			logger.Information("Pulling fake region data.");
			await Task.Delay(100);
			logger.Information("Pulling fake region data successful.");
			return new List<ForeignRegion>()
			{
				new ForeignRegion(1, "Liberecký kraj"),
				new ForeignRegion(2, "Ústecký kraj"),
				new ForeignRegion(3, "Středočeský kraj"),
				new ForeignRegion(4, "Královéhradecký kraj"),
				new ForeignRegion(5, "Pardubický kraj"),
				new ForeignRegion(6, "Kraj Vysočina"),
				new ForeignRegion(7, "Praha"),
				new ForeignRegion(8, "Karlovarský kraj"),
				new ForeignRegion(9, "Plzeňský kraj"),
				new ForeignRegion(10, "Jihočeský kraj"),
				new ForeignRegion(11, "Plzeňský kraj"),
				new ForeignRegion(12, "Olomoucký kraj"),
				new ForeignRegion(13, "Jihomoravský kraj"),
				new ForeignRegion(14, "Zlínský kraj"),
				new ForeignRegion(15, "Moravskoslezský kraj")
			};
		}
#warning TEST METHOD
		public async Task<List<ForeignSport>> GetSports()
		{
			logger.Information("Pulling fake Sport data.");
			await Task.Delay(100);
			logger.Information("Pulling fake Sport data successful.");
			return new List<ForeignSport>()
			{
				new ForeignSport(1, "Fotbal"),
				new ForeignSport(2, "Volejbal"),
				new ForeignSport(3, "Veslování"),
				new ForeignSport(4, "Florbal"),
				new ForeignSport(5, "Atletika"),
				new ForeignSport(6, "Karate"),
			};
		}
#warning TEST METHOD
		public async Task<List<ForeignVenue>> GetVenues()
		{
			logger.Information("Pulling fake Venue data.");
			await Task.Delay(100);
			logger.Information("Pulling fake Venue data successful.");
			return new List<ForeignVenue>()
			{
				new ForeignVenue(1, "Atletický stadion Střelnice", 1, 50.7161666, 15.163021899999, "U Stadionu", "4904/5", "Jablonec nad Nisou"),
				new ForeignVenue(2, "Městská hala", 1, 50.7327979999, 15.1791156999999, "U Přehrady", "4747/20", "Jablonec nad Nisou"),
				new ForeignVenue(3, "Hřiště 1 Mozartova", 1, 50.7389005, 15.1627788000000, "Mozartova", "3668/22", "Jablonec nad Nisou"),
				new ForeignVenue(4, "Hřiště 2 Mozartova", 1, 50.7389005, 15.1627788000000, "Mozartova", "3668/22", "Jablonec nad Nisou"),
				new ForeignVenue(5, "Hřiště 3 Mozartova", 1, 50.7389005, 15.1627788000000, "Mozartova", "3668/22", "Jablonec nad Nisou"),
			};
		}
	}
}