﻿using System;
using System.Threading.Tasks;
using Hangfire;
using ODMWeb.DL;
using ODMWeb.DL.Exceptions;
using ODMWeb.ForeignApiManager;
using ODMWeb.Hangfire.Jobs;
using Serilog;

namespace ODMWeb.Hangfire.RecurringJobs
{
	public class UpdateCompetitionDataTask : BaseHangfireTask<UpdateCompetitionDataTask>
	{
		[AutomaticRetry(DelaysInSeconds = new[] { 60 })]
		public override async Task Run()
		{
			logger.Information("UpdateCompetitionDataTask start.");
			var service = new WebService();
			var data = await service.GetCompetitions();
			try
			{
				await this.unitOfWork.Competitions.UpdateFromForeign(data);
				logger.Information("UpdateCompetitionDataTask successful.");
			}
			catch (SportDependencyNotResolvedException ex)
			{
				logger.Error(ex, "UpdateCompetitionDataTask failed. Sport dependency not resolved.");
				BackgroundJob.Enqueue<UpdateSportDataTask>(t => t.Run());
				logger.Error("Stopping UpdateCompetitionDataTask execution.");
				throw new InvalidOperationException();
			}
		}

		public UpdateCompetitionDataTask(IUnitOfWork unitOfWork) : base(unitOfWork)
		{
		}
	}
}