﻿using System;
using System.Threading.Tasks;
using Hangfire;
using ODMWeb.DL;
using ODMWeb.DL.Exceptions;
using ODMWeb.ForeignApiManager;
using ODMWeb.Hangfire.Jobs;
using Serilog;

namespace ODMWeb.Hangfire.RecurringJobs
{
	public class UpdateEventDataTask : BaseHangfireTask<UpdateEventDataTask>
	{
		public UpdateEventDataTask(IUnitOfWork unitOfWork) : base(unitOfWork)
		{
		}

		[AutomaticRetry(DelaysInSeconds = new[] { 60 })]
		public override async Task Run()
		{
			logger.Information("UpdateEventDataTask start.");
			var service = new WebService();
			var data = await service.GetEvents();

			try
			{
				await this.unitOfWork.Events.UpdateFromForeign(data);
				logger.Information("UpdateEventDataTask successful.");
			}
			catch (AggregateException e)
			{
				foreach (var exception in e.InnerExceptions)
				{
					logger.Error("UpdateEventDataTask multiple dependencies not resolved.");
					if (exception is CompetitionDependencyNotResolvedException)
					{
						BackgroundJob.Enqueue<UpdateCompetitionDataTask>(t => t.Run());
						logger.Error(exception, "UpdateEventDataTask failed. Competition dependency not resolved.");
					}

					if (exception is VenueDependencyNotResolvedException)
					{
						BackgroundJob.Enqueue<UpdateVenueDataTask>(t => t.Run());
						logger.Error(exception, "UpdateEventDataTask failed. Venue dependency not resolved.");
					}
				}
				logger.Error("Stopping UpdateEventDataTask execution.");
				throw new InvalidOperationException();
			}
			catch (CompetitionDependencyNotResolvedException e)
			{
				logger.Error(e, "UpdateEventDataTask failed. Competition dependency not resolved.");

				BackgroundJob.Enqueue<UpdateCompetitionDataTask>(t => t.Run());
				logger.Error("Stopping UpdateEventDataTask execution.");
				throw new InvalidOperationException();
			}
			catch (VenueDependencyNotResolvedException e)
			{
				logger.Error(e, "UpdateEventDataTask failed. Venue dependency not resolved.");

				BackgroundJob.Enqueue<UpdateVenueDataTask>(t => t.Run());
				logger.Error("Stopping UpdateEventDataTask execution.");
				throw new InvalidOperationException();
			}
		}
	}
}